# -*- coding: utf-8 -*-
'''
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
'''

# Import Python Libs
from __future__ import absolute_import, unicode_literals, print_function
import types
from datetime import datetime

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.modules.win_system as win_system


@skipIf(NO_MOCK, NO_MOCK_REASON)
class WinSystemTestCase(TestCase, LoaderModuleMockMixin):
    '''
        Test cases for salt.modules.win_system
    '''
    def setup_loader_modules(self):
        modules_globals = {}
        if win_system.HAS_WIN32NET_MODS is False:
            win32api = types.ModuleType(
                str('win32api')  # future lint: disable=blacklisted-function
            )
            now = datetime.now()
            win32api.GetLocalTime = MagicMock(return_value=[now.year, now.month, now.weekday(),
                                                            now.day, now.hour, now.minute,
                                                            now.second, now.microsecond])
            modules_globals['win32api'] = win32api
        return {win_system: modules_globals}

    def test_halt(self):
        '''
            Test to halt a running system
        '''
        mock = MagicMock(return_value='salt')
        with patch.object(win_system, 'shutdown', mock):
            self.assertEqual(win_system.halt(), 'salt')

    def test_init(self):
        '''
            Test to change the system runlevel on sysV compatible systems
        '''
        self.assertEqual(win_system.init(3),
                         'Not implemented on Windows at this time.')

    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs the w32net library')
    def test_poweroff(self):
        '''
            Test to poweroff a running system
        '''
        mock = MagicMock(return_value='salt')
        with patch.object(win_system, 'shutdown', mock):
            self.assertEqual(win_system.poweroff(), 'salt')

    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs the w32net library')
    def test_reboot(self):
        '''
            Test to reboot the system
        '''
        with patch('salt.modules.win_system.shutdown',
                   MagicMock(return_value=True)) as shutdown:
            self.assertEqual(win_system.reboot(), True)

    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs the w32net library')
    def test_reboot_with_timeout_in_minutes(self):
        '''
            Test to reboot the system with a timeout
        '''
        with patch('salt.modules.win_system.shutdown',
                   MagicMock(return_value=True)) as shutdown:
            self.assertEqual(win_system.reboot(5, in_seconds=False), True)
            shutdown.assert_called_with(timeout=5, in_seconds=False, reboot=True,
                                        only_on_pending_reboot=False)

    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs the w32net library')
    def test_reboot_with_timeout_in_seconds(self):
        '''
            Test to reboot the system with a timeout
        '''
        with patch('salt.modules.win_system.shutdown',
                   MagicMock(return_value=True)) as shutdown:
            self.assertEqual(win_system.reboot(5, in_seconds=True), True)
            shutdown.assert_called_with(timeout=5, in_seconds=True, reboot=True,
                                        only_on_pending_reboot=False)

    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs the w32net library')
    def test_reboot_with_wait(self):
        '''
            Test to reboot the system with a timeout and
            wait for it to finish
        '''
        with patch('salt.modules.win_system.shutdown',
                   MagicMock(return_value=True)), \
                patch('salt.modules.win_system.time.sleep',
                      MagicMock()) as time:
            self.assertEqual(win_system.reboot(wait_for_reboot=True), True)
            time.assert_called_with(330)

    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs the w32net library')
    def test_shutdown(self):
        '''
            Test to shutdown a running system
        '''
        with patch('salt.modules.win_system.win32api.InitiateSystemShutdown',
                   MagicMock()):
            self.assertEqual(win_system.shutdown(), True)

    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs the w32net library')
    def test_shutdown_hard(self):
        '''
            Test to shutdown a running system with no timeout or warning
        '''
        with patch('salt.modules.win_system.shutdown',
                   MagicMock(return_value=True)) as shutdown:
            self.assertEqual(win_system.shutdown_hard(), True)
            shutdown.assert_called_with(timeout=0)

    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs the w32net library')
    def test_set_computer_name(self):
        '''
            Test to set the Windows computer name
        '''
        with patch('salt.modules.win_system.windll.kernel32.SetComputerNameExW',
                   MagicMock(return_value=True)):
            with patch.object(win_system, 'get_computer_name',
                              MagicMock(return_value='salt')):
                with patch.object(win_system, 'get_pending_computer_name',
                                  MagicMock(return_value='salt_new')):
                    self.assertDictEqual(win_system.set_computer_name("salt_new"),
                                         {'Computer Name': {'Current': 'salt',
                                                            'Pending': 'salt_new'}})
        # Test set_computer_name failure
        with patch('salt.modules.win_system.windll.kernel32.SetComputerNameExW',
                   MagicMock(return_value=False)):
            self.assertFalse(win_system.set_computer_name("salt"))

    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs the w32net library')
    def test_get_pending_computer_name(self):
        '''
            Test to get a pending computer name.
        '''
        with patch.object(win_system, 'get_computer_name',
                          MagicMock(return_value='salt')):
            reg_mock = MagicMock(return_value={'vdata': 'salt'})
            with patch.dict(win_system.__salt__, {'reg.read_value': reg_mock}):
                self.assertFalse(win_system.get_pending_computer_name())

            reg_mock = MagicMock(return_value={'vdata': 'salt_pending'})
            with patch.dict(win_system.__salt__, {'reg.read_value': reg_mock}):
                self.assertEqual(win_system.get_pending_computer_name(),
                                 'salt_pending')

    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs the w32net library')
    def test_get_computer_name(self):
        '''
            Test to get the Windows computer name
        '''
        with patch('salt.modules.win_system.win32api.GetComputerNameEx',
                   MagicMock(side_effect=['computer name', ''])):
            self.assertEqual(win_system.get_computer_name(), 'computer name')
            self.assertFalse(win_system.get_computer_name())

    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs the w32net library')
    def test_set_computer_desc(self):
        '''
            Test to set the Windows computer description
        '''
        mock = MagicMock(return_value=True)
        with patch.dict(win_system.__salt__, {'cmd.run': mock}):
            mock = MagicMock(return_value="Salt's comp")
            with patch.object(win_system, 'get_computer_desc', mock):
                self.assertDictEqual(win_system.set_computer_desc(
                                                                  "Salt's comp"
                                                                  ),
                                     {'Computer Description': "Salt's comp"})

    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs the w32net library')
    def test_get_computer_desc(self):
        '''
            Test to get the Windows computer description
        '''
        with patch('salt.modules.win_system.get_system_info',
                   MagicMock(side_effect=[{'description': 'salt description'},
                                          {'description': None}])):
            self.assertEqual(win_system.get_computer_desc(), 'salt description')
            self.assertFalse(win_system.get_computer_desc())

    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs w32net and other windows libraries')
    def test_join_domain(self):
        '''
            Test to join a computer to an Active Directory domain
        '''
        with patch('salt.modules.win_system._join_domain',
                   MagicMock(return_value=0)):
            with patch('salt.modules.win_system.get_domain_workgroup',
                       MagicMock(return_value={'Workgroup': 'Workgroup'})):
                self.assertDictEqual(
                    win_system.join_domain(
                        "saltstack", "salt", "salt@123"),
                        {'Domain': 'saltstack', 'Restart': False})

            with patch('salt.modules.win_system.get_domain_workgroup',
                       MagicMock(return_value={'Domain': 'saltstack'})):
                self.assertEqual(
                    win_system.join_domain("saltstack", "salt", "salt@123"),
                    'Already joined to saltstack')

    def test_get_system_time(self):
        '''
            Test to get system time
        '''
        tm = datetime.strftime(datetime.now(), "%I:%M:%S %p")
        win_tm = win_system.get_system_time()
        try:
            self.assertEqual(win_tm, tm)
        except AssertionError:
            # handle race condition
            import re
            self.assertTrue(re.search(r'^\d{2}:\d{2} \w{2}$', win_tm))

    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs the w32net library')
    def test_set_system_time(self):
        '''
            Test to set system time
        '''
        with patch('salt.modules.win_system.set_system_date_time',
                   MagicMock(side_effect=[False, True])):
            self.assertFalse(win_system.set_system_time("11:31:15 AM"))
            self.assertTrue(win_system.set_system_time("11:31:15 AM"))

    def test_get_system_date(self):
        '''
            Test to get system date
        '''
        date = datetime.strftime(datetime.now(), "%m/%d/%Y")
        self.assertEqual(win_system.get_system_date(), date)

    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs the w32net library')
    def test_set_system_date(self):
        '''
            Test to set system date
        '''
        with patch('salt.modules.win_system.set_system_date_time',
                   MagicMock(side_effect=[False, True])):
            self.assertFalse(win_system.set_system_date("03-28-13"))
            self.assertTrue(win_system.set_system_date("03-28-13"))

    def test_start_time_service(self):
        '''
            Test to start the Windows time service
        '''
        mock = MagicMock(return_value=True)
        with patch.dict(win_system.__salt__, {'service.start': mock}):
            self.assertTrue(win_system.start_time_service())

    def test_stop_time_service(self):
        '''
            Test to stop the windows time service
        '''
        mock = MagicMock(return_value=True)
        with patch.dict(win_system.__salt__, {'service.stop': mock}):
            self.assertTrue(win_system.stop_time_service())

    def test_set_hostname(self):
        '''
            Test setting a new hostname
        '''
        cmd_run_mock = MagicMock(return_value="Method execution successful.")
        get_hostname = MagicMock(return_value="MINION")
        with patch.dict(win_system.__salt__, {'cmd.run': cmd_run_mock}):
            with patch.object(win_system, 'get_hostname', get_hostname):
                win_system.set_hostname("NEW")

        cmd_run_mock.assert_called_once_with(cmd="wmic computersystem where name='MINION' call rename name='NEW'")

    def test_get_hostname(self):
        '''
            Test setting a new hostname
        '''
        cmd_run_mock = MagicMock(return_value="MINION")
        with patch.dict(win_system.__salt__, {'cmd.run': cmd_run_mock}):
            ret = win_system.get_hostname()
            self.assertEqual(ret, "MINION")
        cmd_run_mock.assert_called_once_with(cmd="hostname")



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jeff Schroeder <jeffschroeder@computer.org>`
'''
# Import Python libs
from __future__ import absolute_import, unicode_literals, print_function
import base64

from contextlib import contextmanager


# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch)

# Import Salt Libs
import salt.utils.stringutils
from salt.states import kubernetes
from salt.ext import six


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(kubernetes is False, "Probably Kubernetes client lib is not installed. \
                              Skipping test_kubernetes.py")
class KubernetesTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.states.kubernetes
    '''
    def setup_loader_modules(self):
        return {kubernetes: {'__env__': 'base'}}

    @contextmanager
    def mock_func(self, func_name, return_value, test=False):
        '''
        Mock any of the kubernetes state function return values and set
        the test options.
        '''
        name = 'kubernetes.{0}'.format(func_name)
        mocked = {name: MagicMock(return_value=return_value)}
        with patch.dict(kubernetes.__salt__, mocked) as patched:
            with patch.dict(kubernetes.__opts__, {'test': test}):
                yield patched

    def make_configmap(self, name, namespace='default', data=None):
        return self.make_ret_dict(
            kind='ConfigMap',
            name=name,
            namespace=namespace,
            data=data,
        )

    def make_secret(self, name, namespace='default', data=None):
        secret_data = self.make_ret_dict(
            kind='Secret',
            name=name,
            namespace=namespace,
            data=data,
        )
        # Base64 all of the values just like kubectl does
        for key, value in six.iteritems(secret_data['data']):
            secret_data['data'][key] = base64.b64encode(
                salt.utils.stringutils.to_bytes(value)
            )

        return secret_data

    def make_node_labels(self, name='minikube'):
        return {
            'kubernetes.io/hostname': name,
            'beta.kubernetes.io/os': 'linux',
            'beta.kubernetes.io/arch': 'amd64',
            'failure-domain.beta.kubernetes.io/region': 'us-west-1',
        }

    def make_node(self, name='minikube'):
        node_data = self.make_ret_dict(kind='Node', name='minikube')
        node_data.update({
            'api_version': 'v1',
            'kind': 'Node',
            'metadata': {
                'annotations': {
                    'node.alpha.kubernetes.io/ttl': '0',
                },
                'labels': self.make_node_labels(name=name),
                'name': name,
                'namespace': None,
                'self_link': '/api/v1/nodes/{name}'.format(name=name),
                'uid': '7811b8ae-c1a1-11e7-a55a-0800279fb61e',
            },
            'spec': {
                'external_id': name,
            },
            'status': {},
        })
        return node_data

    def make_namespace(self, name='default'):
        namespace_data = self.make_ret_dict(kind='Namespace', name=name)
        del namespace_data['data']
        namespace_data.update({
            'status': {
                'phase': 'Active',
            },
            'spec': {
                'finalizers': ['kubernetes'],
            },
            'metadata': {
                'name': name,
                'namespace': None,
                'labels': None,
                'self_link': '/api/v1/namespaces/{namespace}'.format(
                    namespace=name,
                ),
                'annotations': None,
                'uid': '752fceeb-c1a1-11e7-a55a-0800279fb61e',
            },
        })
        return namespace_data

    def make_ret_dict(self, kind, name, namespace=None, data=None):
        '''
        Make a minimal example configmap or secret for using in mocks
        '''

        assert kind in ('Secret', 'ConfigMap', 'Namespace', 'Node')

        if data is None:
            data = {}

        self_link = '/api/v1/namespaces/{namespace}/{kind}s/{name}'.format(
            namespace=namespace,
            kind=kind.lower(),
            name=name,
        )

        return_data = {
            'kind': kind,
            'data': data,
            'api_version': 'v1',
            'metadata': {
                'name': name,
                'labels': None,
                'namespace': namespace,
                'self_link': self_link,
                'annotations': {
                    'kubernetes.io/change-cause': 'salt-call state.apply',
                },
            },
        }
        return return_data

    def test_configmap_present__fail(self):
        error = kubernetes.configmap_present(
            name='testme',
            data={1: 1},
            source='salt://beyond/oblivion.jinja',
        )
        self.assertDictEqual(
            {
                'changes': {},
                'result': False,
                'name': 'testme',
                'comment': "'source' cannot be used in combination with 'data'",
            },
            error,
        )

    def test_configmap_present__create_test_true(self):
        # Create a new configmap with test=True
        with self.mock_func('show_configmap', return_value=None, test=True):
            ret = kubernetes.configmap_present(
                name='example',
                data={'example.conf': '# empty config file'},
            )
            self.assertDictEqual(
                {
                    'comment': 'The configmap is going to be created',
                    'changes': {},
                    'name': 'example',
                    'result': None,
                },
                ret,
            )

    def test_configmap_present__create(self):
        # Create a new configmap
        with self.mock_func('show_configmap', return_value=None):
            cm = self.make_configmap(
                name='test',
                namespace='default',
                data={'foo': 'bar'},
            )
            with self.mock_func('create_configmap', return_value=cm):
                actual = kubernetes.configmap_present(
                    name='test',
                    data={'foo': 'bar'},
                )
                self.assertDictEqual(
                    {
                        'comment': '',
                        'changes': {'data': {'foo': 'bar'}},
                        'name': 'test',
                        'result': True,
                    },
                    actual,
                )

    def test_configmap_present__create_no_data(self):
        # Create a new configmap with no 'data' attribute
        with self.mock_func('show_configmap', return_value=None):
            cm = self.make_configmap(
                name='test',
                namespace='default',
            )
            with self.mock_func('create_configmap', return_value=cm):
                actual = kubernetes.configmap_present(name='test')
                self.assertDictEqual(
                    {
                        'comment': '',
                        'changes': {'data': {}},
                        'name': 'test',
                        'result': True,
                    },
                    actual,
                )

    def test_configmap_present__replace_test_true(self):
        cm = self.make_configmap(
            name='settings',
            namespace='saltstack',
            data={'foobar.conf': '# Example configuration'},
        )
        with self.mock_func('show_configmap', return_value=cm, test=True):
            ret = kubernetes.configmap_present(
                name='settings',
                namespace='saltstack',
                data={'foobar.conf': '# Example configuration'},
            )
            self.assertDictEqual(
                {
                    'comment': 'The configmap is going to be replaced',
                    'changes': {},
                    'name': 'settings',
                    'result': None,
                },
                ret,
            )

    def test_configmap_present__replace(self):
        cm = self.make_configmap(name='settings', data={'action': 'make=war'})
        # Replace an existing configmap
        with self.mock_func('show_configmap', return_value=cm):
            new_cm = cm.copy()
            new_cm.update({
                'data': {'action': 'make=peace'},
            })
            with self.mock_func('replace_configmap', return_value=new_cm):
                actual = kubernetes.configmap_present(
                    name='settings',
                    data={'action': 'make=peace'},
                )
                self.assertDictEqual(
                    {
                        'comment': 'The configmap is already present. Forcing recreation',
                        'changes': {
                            'data': {
                                'action': 'make=peace',
                            },
                        },
                        'name': 'settings',
                        'result': True,
                    },
                    actual,
                )

    def test_configmap_absent__noop_test_true(self):
        # Nothing to delete with test=True
        with self.mock_func('show_configmap', return_value=None, test=True):
            actual = kubernetes.configmap_absent(name='NOT_FOUND')
            self.assertDictEqual(
                {
                    'comment': 'The configmap does not exist',
                    'changes': {},
                    'name': 'NOT_FOUND',
                    'result': None,
                },
                actual,
            )

    def test_configmap_absent__test_true(self):
        # Configmap exists with test=True
        cm = self.make_configmap(name='deleteme', namespace='default')
        with self.mock_func('show_configmap', return_value=cm, test=True):
            actual = kubernetes.configmap_absent(name='deleteme')
            self.assertDictEqual(
                {
                    'comment': 'The configmap is going to be deleted',
                    'changes': {},
                    'name': 'deleteme',
                    'result': None,
                },
                actual,
            )

    def test_configmap_absent__noop(self):
        # Nothing to delete
        with self.mock_func('show_configmap', return_value=None):
            actual = kubernetes.configmap_absent(name='NOT_FOUND')
            self.assertDictEqual(
                {
                    'comment': 'The configmap does not exist',
                    'changes': {},
                    'name': 'NOT_FOUND',
                    'result': True,
                },
                actual,
            )

    def test_configmap_absent(self):
        # Configmap exists, delete it!
        cm = self.make_configmap(name='deleteme', namespace='default')
        with self.mock_func('show_configmap', return_value=cm):
            # The return from this module isn't used in the state
            with self.mock_func('delete_configmap', return_value={}):
                actual = kubernetes.configmap_absent(name='deleteme')
                self.assertDictEqual(
                    {
                        'comment': 'ConfigMap deleted',
                        'changes': {
                            'kubernetes.configmap': {
                                'new': 'absent',
                                'old': 'present',
                            },
                        },
                        'name': 'deleteme',
                        'result': True,
                    },
                    actual,
                )

    def test_secret_present__fail(self):
        actual = kubernetes.secret_present(
            name='sekret',
            data={'password': 'monk3y'},
            source='salt://nope.jinja',
        )
        self.assertDictEqual(
            {
                'changes': {},
                'result': False,
                'name': 'sekret',
                'comment': "'source' cannot be used in combination with 'data'",
            },
            actual,
        )

    def test_secret_present__exists_test_true(self):
        secret = self.make_secret(name='sekret')
        new_secret = secret.copy()
        new_secret.update({
            'data': {'password': 'uncle'},
        })
        # Secret exists already and needs replacing with test=True
        with self.mock_func('show_secret', return_value=secret):
            with self.mock_func('replace_secret', return_value=new_secret, test=True):
                actual = kubernetes.secret_present(
                    name='sekret',
                    data={'password': 'uncle'},
                )
                self.assertDictEqual(
                    {
                        'changes': {},
                        'result': None,
                        'name': 'sekret',
                        'comment': 'The secret is going to be replaced',
                    },
                    actual,
                )

    def test_secret_present__exists(self):
        # Secret exists and gets replaced
        secret = self.make_secret(name='sekret', data={'password': 'booyah'})
        with self.mock_func('show_secret', return_value=secret):
            with self.mock_func('replace_secret', return_value=secret):
                actual = kubernetes.secret_present(
                    name='sekret',
                    data={'password': 'booyah'},
                )
                self.assertDictEqual(
                    {
                        'changes': {'data': ['password']},
                        'result': True,
                        'name': 'sekret',
                        'comment': "The secret is already present. Forcing recreation",
                    },
                    actual,
                )

    def test_secret_present__create(self):
        # Secret exists and gets replaced
        secret = self.make_secret(name='sekret', data={'password': 'booyah'})
        with self.mock_func('show_secret', return_value=None):
            with self.mock_func('create_secret', return_value=secret):
                actual = kubernetes.secret_present(
                    name='sekret',
                    data={'password': 'booyah'},
                )
                self.assertDictEqual(
                    {
                        'changes': {'data': ['password']},
                        'result': True,
                        'name': 'sekret',
                        'comment': '',
                    },
                    actual,
                )

    def test_secret_present__create_no_data(self):
        # Secret exists and gets replaced
        secret = self.make_secret(name='sekret')
        with self.mock_func('show_secret', return_value=None):
            with self.mock_func('create_secret', return_value=secret):
                actual = kubernetes.secret_present(name='sekret')
                self.assertDictEqual(
                    {
                        'changes': {'data': []},
                        'result': True,
                        'name': 'sekret',
                        'comment': '',
                    },
                    actual,
                )

    def test_secret_present__create_test_true(self):
        # Secret exists and gets replaced with test=True
        secret = self.make_secret(name='sekret')
        with self.mock_func('show_secret', return_value=None):
            with self.mock_func('create_secret', return_value=secret, test=True):
                actual = kubernetes.secret_present(name='sekret')
                self.assertDictEqual(
                    {
                        'changes': {},
                        'result': None,
                        'name': 'sekret',
                        'comment': 'The secret is going to be created',
                    },
                    actual,
                )

    def test_secret_absent__noop_test_true(self):
        with self.mock_func('show_secret', return_value=None, test=True):
            actual = kubernetes.secret_absent(name='sekret')
            self.assertDictEqual(
                {
                    'changes': {},
                    'result': None,
                    'name': 'sekret',
                    'comment': 'The secret does not exist',
                },
                actual,
            )

    def test_secret_absent__noop(self):
        with self.mock_func('show_secret', return_value=None):
            actual = kubernetes.secret_absent(name='passwords')
            self.assertDictEqual(
                {
                    'changes': {},
                    'result': True,
                    'name': 'passwords',
                    'comment': 'The secret does not exist',
                },
                actual,
            )

    def test_secret_absent__delete_test_true(self):
        secret = self.make_secret(name='credentials', data={'redis': 'letmein'})
        with self.mock_func('show_secret', return_value=secret):
            with self.mock_func('delete_secret', return_value=secret, test=True):
                actual = kubernetes.secret_absent(name='credentials')
                self.assertDictEqual(
                    {
                        'changes': {},
                        'result': None,
                        'name': 'credentials',
                        'comment': 'The secret is going to be deleted',
                    },
                    actual,
                )

    def test_secret_absent__delete(self):
        secret = self.make_secret(name='foobar', data={'redis': 'letmein'})
        deleted = {
            'status': None,
            'kind': 'Secret',
            'code': None,
            'reason': None,
            'details': None,
            'message': None,
            'api_version': 'v1',
            'metadata': {
                'self_link': '/api/v1/namespaces/default/secrets/foobar',
                'resource_version': '30292',
            },
        }
        with self.mock_func('show_secret', return_value=secret):
            with self.mock_func('delete_secret', return_value=deleted):
                actual = kubernetes.secret_absent(name='foobar')
                self.assertDictEqual(
                    {
                        'changes': {
                            'kubernetes.secret': {
                                'new': 'absent',
                                'old': 'present',
                            },
                        },
                        'result': True,
                        'name': 'foobar',
                        'comment': 'Secret deleted',
                    },
                    actual,
                )

    def test_node_label_present__add_test_true(self):
        labels = self.make_node_labels()
        with self.mock_func('node_labels', return_value=labels, test=True):
            actual = kubernetes.node_label_present(
                name='com.zoo-animal',
                node='minikube',
                value='monkey',
            )
            self.assertDictEqual(
                {
                    'changes': {},
                    'result': None,
                    'name': 'com.zoo-animal',
                    'comment': 'The label is going to be set',
                },
                actual,
            )

    def test_node_label_present__add(self):
        node_data = self.make_node()
        # Remove some of the defaults to make it simpler
        node_data['metadata']['labels'] = {
            'beta.kubernetes.io/os': 'linux',
        }
        labels = node_data['metadata']['labels']

        with self.mock_func('node_labels', return_value=labels):
            with self.mock_func('node_add_label', return_value=node_data):
                actual = kubernetes.node_label_present(
                    name='failure-domain.beta.kubernetes.io/zone',
                    node='minikube',
                    value='us-central1-a',
                )
                self.assertDictEqual(
                    {
                        'comment': '',
                        'changes': {
                            'minikube.failure-domain.beta.kubernetes.io/zone': {
                                'new': {
                                    'failure-domain.beta.kubernetes.io/zone': 'us-central1-a',
                                    'beta.kubernetes.io/os': 'linux'
                                },
                                'old': {
                                    'beta.kubernetes.io/os': 'linux',
                                },
                            },
                        },
                        'name': 'failure-domain.beta.kubernetes.io/zone',
                        'result': True,
                    },
                    actual,
                )

    def test_node_label_present__already_set(self):
        node_data = self.make_node()
        labels = node_data['metadata']['labels']
        with self.mock_func('node_labels', return_value=labels):
            with self.mock_func('node_add_label', return_value=node_data):
                actual = kubernetes.node_label_present(
                    name='failure-domain.beta.kubernetes.io/region',
                    node='minikube',
                    value='us-west-1',
                )
                self.assertDictEqual(
                    {
                        'changes': {},
                        'result': True,
                        'name': 'failure-domain.beta.kubernetes.io/region',
                        'comment': 'The label is already set and has the specified value',
                    },
                    actual,
                )

    def test_node_label_present__update_test_true(self):
        node_data = self.make_node()
        labels = node_data['metadata']['labels']
        with self.mock_func('node_labels', return_value=labels):
            with self.mock_func('node_add_label', return_value=node_data, test=True):
                actual = kubernetes.node_label_present(
                    name='failure-domain.beta.kubernetes.io/region',
                    node='minikube',
                    value='us-east-1',
                )
                self.assertDictEqual(
                    {
                        'changes': {},
                        'result': None,
                        'name': 'failure-domain.beta.kubernetes.io/region',
                        'comment': 'The label is going to be updated',
                    },
                    actual,
                )

    def test_node_label_present__update(self):
        node_data = self.make_node()
        # Remove some of the defaults to make it simpler
        node_data['metadata']['labels'] = {
            'failure-domain.beta.kubernetes.io/region': 'us-west-1',
        }
        labels = node_data['metadata']['labels']
        with self.mock_func('node_labels', return_value=labels):
            with self.mock_func('node_add_label', return_value=node_data):
                actual = kubernetes.node_label_present(
                    name='failure-domain.beta.kubernetes.io/region',
                    node='minikube',
                    value='us-east-1',
                )
                self.assertDictEqual(
                    {
                        'changes': {
                            'minikube.failure-domain.beta.kubernetes.io/region': {
                                'new': {'failure-domain.beta.kubernetes.io/region': 'us-east-1'},
                                'old': {'failure-domain.beta.kubernetes.io/region': 'us-west-1'},
                            }
                        },
                        'result': True,
                        'name': 'failure-domain.beta.kubernetes.io/region',
                        'comment': 'The label is already set, changing the value',
                    },
                    actual,
                )

    def test_node_label_absent__noop_test_true(self):
        labels = self.make_node_labels()
        with self.mock_func('node_labels', return_value=labels, test=True):
            actual = kubernetes.node_label_absent(
                name='non-existent-label',
                node='minikube',
            )
            self.assertDictEqual(
                {
                    'changes': {},
                    'result': None,
                    'name': 'non-existent-label',
                    'comment': 'The label does not exist',
                },
                actual
            )

    def test_node_label_absent__noop(self):
        labels = self.make_node_labels()
        with self.mock_func('node_labels', return_value=labels):
            actual = kubernetes.node_label_absent(
                name='non-existent-label',
                node='minikube',
            )
            self.assertDictEqual(
                {
                    'changes': {},
                    'result': True,
                    'name': 'non-existent-label',
                    'comment': 'The label does not exist',
                },
                actual
            )

    def test_node_label_absent__delete_test_true(self):
        labels = self.make_node_labels()
        with self.mock_func('node_labels', return_value=labels, test=True):
            actual = kubernetes.node_label_absent(
                name='failure-domain.beta.kubernetes.io/region',
                node='minikube',
            )
            self.assertDictEqual(
                {
                    'changes': {},
                    'result': None,
                    'name': 'failure-domain.beta.kubernetes.io/region',
                    'comment': 'The label is going to be deleted',
                },
                actual
            )

    def test_node_label_absent__delete(self):
        node_data = self.make_node()
        labels = node_data['metadata']['labels'].copy()

        del node_data['metadata']['labels']['failure-domain.beta.kubernetes.io/region']

        with self.mock_func('node_labels', return_value=labels):
            with self.mock_func('node_remove_label', return_value=node_data):
                actual = kubernetes.node_label_absent(
                    name='failure-domain.beta.kubernetes.io/region',
                    node='minikube',
                )
                self.assertDictEqual(
                    {
                        'result': True,
                        'changes': {
                            'kubernetes.node_label': {
                                'new': 'absent',
                                'old': 'present',
                            }
                        },
                        'comment': 'Label removed from node',
                        'name': 'failure-domain.beta.kubernetes.io/region',
                    },
                    actual
                )

    def test_namespace_present__create_test_true(self):
        with self.mock_func('show_namespace', return_value=None, test=True):
            actual = kubernetes.namespace_present(name='saltstack')
            self.assertDictEqual(
                {
                    'changes': {},
                    'result': None,
                    'name': 'saltstack',
                    'comment': 'The namespace is going to be created',
                },
                actual
            )

    def test_namespace_present__create(self):
        namespace_data = self.make_namespace(name='saltstack')
        with self.mock_func('show_namespace', return_value=None):
            with self.mock_func('create_namespace', return_value=namespace_data):
                actual = kubernetes.namespace_present(name='saltstack')
                self.assertDictEqual(
                    {
                        'changes': {
                            'namespace': {
                                'new': namespace_data,
                                'old': {},
                            },
                        },
                        'result': True,
                        'name': 'saltstack',
                        'comment': '',
                    },
                    actual
                )

    def test_namespace_present__noop_test_true(self):
        namespace_data = self.make_namespace(name='saltstack')
        with self.mock_func('show_namespace', return_value=namespace_data, test=True):
            actual = kubernetes.namespace_present(name='saltstack')
            self.assertDictEqual(
                {
                    'changes': {},
                    'result': None,
                    'name': 'saltstack',
                    'comment': 'The namespace already exists',
                },
                actual
            )

    def test_namespace_present__noop(self):
        namespace_data = self.make_namespace(name='saltstack')
        with self.mock_func('show_namespace', return_value=namespace_data):
            actual = kubernetes.namespace_present(name='saltstack')
            self.assertDictEqual(
                {
                    'changes': {},
                    'result': True,
                    'name': 'saltstack',
                    'comment': 'The namespace already exists',
                },
                actual
            )

    def test_namespace_absent__noop_test_true(self):
        with self.mock_func('show_namespace', return_value=None, test=True):
            actual = kubernetes.namespace_absent(name='salt')
            self.assertDictEqual(
                {
                    'changes': {},
                    'result': None,
                    'name': 'salt',
                    'comment': 'The namespace does not exist',
                },
                actual
            )

    def test_namespace_absent__noop(self):
        with self.mock_func('show_namespace', return_value=None):
            actual = kubernetes.namespace_absent(name='salt')
            self.assertDictEqual(
                {
                    'changes': {},
                    'result': True,
                    'name': 'salt',
                    'comment': 'The namespace does not exist',
                },
                actual
            )

    def test_namespace_absent__delete_test_true(self):
        namespace_data = self.make_namespace(name='salt')
        with self.mock_func('show_namespace', return_value=namespace_data, test=True):
            actual = kubernetes.namespace_absent(name='salt')
            self.assertDictEqual(
                {
                    'changes': {},
                    'result': None,
                    'name': 'salt',
                    'comment': 'The namespace is going to be deleted',
                },
                actual
            )

    def test_namespace_absent__delete_code_200(self):
        namespace_data = self.make_namespace(name='salt')
        deleted = namespace_data.copy()
        deleted['code'] = 200
        deleted.update({
            'code': 200,
            'message': None,
        })
        with self.mock_func('show_namespace', return_value=namespace_data):
            with self.mock_func('delete_namespace', return_value=deleted):
                actual = kubernetes.namespace_absent(name='salt')
                self.assertDictEqual(
                    {
                        'changes': {
                            'kubernetes.namespace': {
                                'new': 'absent',
                                'old': 'present',
                            }
                        },
                        'result': True,
                        'name': 'salt',
                        'comment': 'Terminating',
                    },
                    actual
                )

    def test_namespace_absent__delete_status_terminating(self):
        namespace_data = self.make_namespace(name='salt')
        deleted = namespace_data.copy()
        deleted.update({
            'code': None,
            'status': 'Terminating namespace',
            'message': 'Terminating this shizzzle yo',
        })
        with self.mock_func('show_namespace', return_value=namespace_data):
            with self.mock_func('delete_namespace', return_value=deleted):
                actual = kubernetes.namespace_absent(name='salt')
                self.assertDictEqual(
                    {
                        'changes': {
                            'kubernetes.namespace': {
                                'new': 'absent',
                                'old': 'present',
                            }
                        },
                        'result': True,
                        'name': 'salt',
                        'comment': 'Terminating this shizzzle yo',
                    },
                    actual
                )

    def test_namespace_absent__delete_status_phase_terminating(self):
        # This is what kubernetes 1.8.0 looks like when deleting namespaces
        namespace_data = self.make_namespace(name='salt')
        deleted = namespace_data.copy()
        deleted.update({
            'code': None,
            'message': None,
            'status': {'phase': 'Terminating'},
        })
        with self.mock_func('show_namespace', return_value=namespace_data):
            with self.mock_func('delete_namespace', return_value=deleted):
                actual = kubernetes.namespace_absent(name='salt')
                self.assertDictEqual(
                    {
                        'changes': {
                            'kubernetes.namespace': {
                                'new': 'absent',
                                'old': 'present',
                            }
                        },
                        'result': True,
                        'name': 'salt',
                        'comment': 'Terminating',
                    },
                    actual
                )

    def test_namespace_absent__delete_error(self):
        namespace_data = self.make_namespace(name='salt')
        deleted = namespace_data.copy()
        deleted.update({
            'code': 418,
            'message': 'I\' a teapot!',
            'status': None,
        })
        with self.mock_func('show_namespace', return_value=namespace_data):
            with self.mock_func('delete_namespace', return_value=deleted):
                actual = kubernetes.namespace_absent(name='salt')
                self.assertDictEqual(
                    {
                        'changes': {},
                        'result': False,
                        'name': 'salt',
                        'comment': 'Something went wrong, response: {0}'.format(
                            deleted,
                        ),
                    },
                    actual
                )



# -*- coding: utf-8 -*-
'''
    :codeauthor: Mike Place <mp@saltstack.com>
'''

# Import python libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, MagicMock, patch

from salt.ext.six.moves import StringIO
import salt.modules.cron as cronmod
import salt.states.cron as cron

STUB_USER = 'root'
STUB_PATH = '/tmp'

STUB_CRON_TIMESTAMP = {
    'minute': '1',
    'hour': '2',
    'daymonth': '3',
    'month': '4',
    'dayweek': '5'}

STUB_SIMPLE_RAW_CRON = '5 0 * * * /tmp/no_script.sh'
STUB_SIMPLE_CRON_DICT = {
    'pre': ['5 0 * * * /tmp/no_script.sh'],
    'crons': [],
    'env': [],
    'special': []}

CRONTAB = StringIO()


def get_crontab(*args, **kw):
    return CRONTAB.getvalue()


def set_crontab(val):
    CRONTAB.seek(0)
    CRONTAB.truncate(0)
    CRONTAB.write(val)


def write_crontab(*args, **kw):
    set_crontab('\n'.join(
        [a.strip() for a in args[1]]))
    return {
        'retcode': False,
    }


@skipIf(NO_MOCK, NO_MOCK_REASON)
class CronTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        loader_gloabals = {
            '__low__': {'__id__': 'noo'},
            '__grains__': {
                'os': 'Debian',
                'os_family': 'Debian',
            },
            '__opts__': {'test': False},
            '__salt__': {
                'cmd.run_all': MagicMock(return_value={
                    'pid': 5,
                    'retcode': 0,
                    'stderr': '',
                    'stdout': ''}),
                'cron.list_tab': cronmod.list_tab,
                'cron.rm_job': cronmod.rm_job,
                'cron.set_job': cronmod.set_job,
            }
        }
        return {cron: loader_gloabals, cronmod: loader_gloabals}

    def setUp(self):
        super(CronTestCase, self).setUp()
        set_crontab('')
        for mod, mock in (('salt.modules.cron.raw_cron', MagicMock(side_effect=get_crontab)),
                          ('salt.modules.cron._write_cron_lines', MagicMock(side_effect=write_crontab))):
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)
        self.addCleanup(set_crontab, '')

    def test_present(self):
        cron.present(
            name='foo',
            hour='1',
            identifier='1',
            user='root')
        self.assertMultiLineEqual(
            get_crontab(),
            '# Lines below here are managed by Salt, do not edit\n'
            '# SALT_CRON_IDENTIFIER:1\n'
            '* 1 * * * foo')
        cron.present(
            name='foo',
            hour='2',
            identifier='1',
            user='root')
        self.assertMultiLineEqual(
            get_crontab(),
            '# Lines below here are managed by Salt, do not edit\n'
            '# SALT_CRON_IDENTIFIER:1\n'
            '* 2 * * * foo')
        cron.present(
            name='cmd1',
            minute='0',
            comment='Commented cron job',
            commented=True,
            identifier='commented_1',
            user='root')
        self.assertMultiLineEqual(
            get_crontab(),
            '# Lines below here are managed by Salt, do not edit\n'
            '# SALT_CRON_IDENTIFIER:1\n'
            '* 2 * * * foo\n'
            '# Commented cron job SALT_CRON_IDENTIFIER:commented_1\n'
            '#DISABLED#0 * * * * cmd1')
        cron.present(
            name='foo',
            hour='2',
            identifier='2',
            user='root')
        self.assertMultiLineEqual(
            get_crontab(),
            '# Lines below here are managed by Salt, do not edit\n'
            '# SALT_CRON_IDENTIFIER:1\n'
            '* 2 * * * foo\n'
            '# Commented cron job SALT_CRON_IDENTIFIER:commented_1\n'
            '#DISABLED#0 * * * * cmd1\n'
            '# SALT_CRON_IDENTIFIER:2\n'
            '* 2 * * * foo')
        cron.present(
            name='cmd2',
            commented=True,
            identifier='commented_2',
            user='root')
        self.assertMultiLineEqual(
            get_crontab(),
            '# Lines below here are managed by Salt, do not edit\n'
            '# SALT_CRON_IDENTIFIER:1\n'
            '* 2 * * * foo\n'
            '# Commented cron job SALT_CRON_IDENTIFIER:commented_1\n'
            '#DISABLED#0 * * * * cmd1\n'
            '# SALT_CRON_IDENTIFIER:2\n'
            '* 2 * * * foo\n'
            '# SALT_CRON_IDENTIFIER:commented_2\n'
            '#DISABLED#* * * * * cmd2')
        set_crontab(
            '# Lines below here are managed by Salt, do not edit\n'
            '# SALT_CRON_IDENTIFIER:1\n'
            '* 2 * * * foo\n'
            '# SALT_CRON_IDENTIFIER:2\n'
            '* 2 * * * foo\n'
            '* 2 * * * foo\n'
        )
        cron.present(
            name='foo',
            hour='2',
            user='root',
            identifier=None)
        self.assertEqual(
            get_crontab(),
            ('# Lines below here are managed by Salt, do not edit\n'
             '# SALT_CRON_IDENTIFIER:1\n'
             '* 2 * * * foo\n'
             '# SALT_CRON_IDENTIFIER:2\n'
             '* 2 * * * foo\n'
             '* 2 * * * foo'))

    def test_remove(self):
        with patch.dict(cron.__opts__, {'test': True}):
            set_crontab(
                '# Lines below here are managed by Salt, do not edit\n'
                '# SALT_CRON_IDENTIFIER:1\n'
                '* 1 * * * foo')
            result = cron.absent(name='bar', identifier='1')
            self.assertEqual(
                result,
                {'changes': {},
                 'comment': 'Cron bar is set to be removed',
                 'name': 'bar',
                 'result': None}
            )
            self.assertEqual(
                get_crontab(),
                '# Lines below here are managed by Salt, do not edit\n'
                '# SALT_CRON_IDENTIFIER:1\n'
                '* 1 * * * foo')
        with patch.dict(cron.__opts__, {'test': False}):
            set_crontab(
                '# Lines below here are managed by Salt, do not edit\n'
                '# SALT_CRON_IDENTIFIER:1\n'
                '* 1 * * * foo')
            cron.absent(name='bar', identifier='1')
            self.assertEqual(
                get_crontab(),
                '# Lines below here are managed by Salt, do not edit'
            )
            set_crontab(
                '# Lines below here are managed by Salt, do not edit\n'
                '* * * * * foo')
            cron.absent(name='bar', identifier='1')
            self.assertEqual(
                get_crontab(),
                '# Lines below here are managed by Salt, do not edit\n'
                '* * * * * foo'
            )
            # old behavior, do not remove with identifier set and
            # even if command match !
            set_crontab(
                '# Lines below here are managed by Salt, do not edit\n'
                '* * * * * foo')
            cron.absent(name='foo', identifier='1')
            self.assertEqual(
                get_crontab(),
                '# Lines below here are managed by Salt, do not edit'
            )
            # old behavior, remove if no identifier and command match
            set_crontab(
                '# Lines below here are managed by Salt, do not edit\n'
                '* * * * * foo')
            cron.absent(name='foo')
            self.assertEqual(
                get_crontab(),
                '# Lines below here are managed by Salt, do not edit'
            )

    def test_multiline_comments_are_updated(self):
        set_crontab(
            '# Lines below here are managed by Salt, do not edit\n'
            '# First crontab - single line comment SALT_CRON_IDENTIFIER:1\n'
            '* 1 * * * foo'
        )
        cron.present(
            name='foo',
            hour='1',
            comment='First crontab\nfirst multi-line comment\n',
            identifier='1',
            user='root')
        cron.present(
            name='foo',
            hour='1',
            comment='First crontab\nsecond multi-line comment\n',
            identifier='1',
            user='root')
        cron.present(
            name='foo',
            hour='1',
            comment='Second crontab\nmulti-line comment\n',
            identifier='2',
            user='root')
        self.assertEqual(
            get_crontab(),
            '# Lines below here are managed by Salt, do not edit\n'
            '# First crontab\n'
            '# second multi-line comment\n'
            '#  SALT_CRON_IDENTIFIER:1\n'
            '* 1 * * * foo\n'
            '# Second crontab\n'
            '# multi-line comment\n'
            '#  SALT_CRON_IDENTIFIER:2\n'
            '* 1 * * * foo')

    def test_existing_unmanaged_jobs_are_made_managed(self):
        set_crontab(
            '# Lines below here are managed by Salt, do not edit\n'
            '0 2 * * * foo'
        )
        ret = cron._check_cron('root', 'foo', hour='2', minute='0')
        self.assertEqual(ret, 'present')
        ret = cron.present('foo', 'root', minute='0', hour='2')
        self.assertEqual(ret['changes'], {'root': 'foo'})
        self.assertEqual(ret['comment'], 'Cron foo updated')
        self.assertEqual(
            get_crontab(),
            '# Lines below here are managed by Salt, do not edit\n'
            '# SALT_CRON_IDENTIFIER:foo\n'
            '0 2 * * * foo')
        ret = cron.present('foo', 'root', minute='0', hour='2')
        self.assertEqual(ret['changes'], {})
        self.assertEqual(ret['comment'], 'Cron foo already present')

    def test_existing_noid_jobs_are_updated_with_identifier(self):
        set_crontab(
            '# Lines below here are managed by Salt, do not edit\n'
            '# SALT_CRON_IDENTIFIER:NO ID SET\n'
            '1 * * * * foo'
        )
        ret = cron._check_cron('root', 'foo', minute=1)
        self.assertEqual(ret, 'present')
        ret = cron.present('foo', 'root', minute=1)
        self.assertEqual(ret['changes'], {'root': 'foo'})
        self.assertEqual(ret['comment'], 'Cron foo updated')
        self.assertEqual(
            get_crontab(),
            '# Lines below here are managed by Salt, do not edit\n'
            '# SALT_CRON_IDENTIFIER:foo\n'
            '1 * * * * foo')

    def test_existing_duplicate_unmanaged_jobs_are_merged_and_given_id(self):
        set_crontab(
            '# Lines below here are managed by Salt, do not edit\n'
            '0 2 * * * foo\n'
            '0 2 * * * foo'
        )
        ret = cron._check_cron('root', 'foo', hour='2', minute='0')
        self.assertEqual(ret, 'present')
        ret = cron.present('foo', 'root', minute='0', hour='2')
        self.assertEqual(ret['changes'], {'root': 'foo'})
        self.assertEqual(ret['comment'], 'Cron foo updated')
        self.assertEqual(
            get_crontab(),
            '# Lines below here are managed by Salt, do not edit\n'
            '# SALT_CRON_IDENTIFIER:foo\n'
            '0 2 * * * foo')
        ret = cron.present('foo', 'root', minute='0', hour='2')
        self.assertEqual(ret['changes'], {})
        self.assertEqual(ret['comment'], 'Cron foo already present')



# -*- coding: utf-8 -*-
'''
    tests.unit.xmlutil_test
    ~~~~~~~~~~~~~~~~~~~~
'''
from __future__ import absolute_import, print_function, unicode_literals
# Import Salt Testing libs
from tests.support.unit import TestCase

# Import Salt libs
from salt._compat import ElementTree as ET
import salt.utils.xmlutil as xml


class XMLUtilTestCase(TestCase):
    '''
    Tests that salt.utils.xmlutil properly parses XML data and returns as a properly formatted
    dictionary. The default method of parsing will ignore attributes and return only the child
    items. The full method will include parsing attributes.
    '''

    def setUp(self):

        # Populate our use cases for specific XML formats.
        self.cases = {
            'a': {
                'xml': '<parent>data</parent>',
                'legacy': {'parent': 'data'},
                'full': 'data'
            },
            'b': {
                'xml': '<parent value="data">data</parent>',
                'legacy': {'parent': 'data'},
                'full': {'parent': 'data', 'value': 'data'}
            },
            'c': {
                'xml': '<parent><child>data</child><child value="data">data</child>'
                       '<child value="data"/><child/></parent>',
                'legacy': {'child': ['data', {'child': 'data'}, {'child': None}, {'child': None}]},
                'full': {'child': ['data', {'child': 'data', 'value': 'data'}, {'value': 'data'}, None]}
            },
            'd': {
                'xml': '<parent value="data" another="data"><child>data</child></parent>',
                'legacy': {'child': 'data'},
                'full': {'child': 'data', 'another': 'data', 'value': 'data'}
            },
            'e': {
                'xml': '<parent value="data" another="data"><child value="data">data</child></parent>',
                'legacy': {'child': 'data'},
                'full': {'child': {'child': 'data', 'value': 'data'}, 'another': 'data', 'value': 'data'}
            },
            'f': {
                'xml': '<parent><child><sub-child value="data">data</sub-child></child>'
                       '<child>data</child></parent>',
                'legacy': {'child': [{'sub-child': 'data'}, {'child': 'data'}]},
                'full': {'child': [{'sub-child': {'value': 'data', 'sub-child': 'data'}}, 'data']}
            },
        }

    def test_xml_case_a(self):
        xmldata = ET.fromstring(self.cases['a']['xml'])
        defaultdict = xml.to_dict(xmldata)
        self.assertEqual(defaultdict, self.cases['a']['legacy'])

    def test_xml_case_a_legacy(self):
        xmldata = ET.fromstring(self.cases['a']['xml'])
        defaultdict = xml.to_dict(xmldata, False)
        self.assertEqual(defaultdict, self.cases['a']['legacy'])

    def test_xml_case_a_full(self):
        xmldata = ET.fromstring(self.cases['a']['xml'])
        defaultdict = xml.to_dict(xmldata, True)
        self.assertEqual(defaultdict, self.cases['a']['full'])

    def test_xml_case_b(self):
        xmldata = ET.fromstring(self.cases['b']['xml'])
        defaultdict = xml.to_dict(xmldata)
        self.assertEqual(defaultdict, self.cases['b']['legacy'])

    def test_xml_case_b_legacy(self):
        xmldata = ET.fromstring(self.cases['b']['xml'])
        defaultdict = xml.to_dict(xmldata, False)
        self.assertEqual(defaultdict, self.cases['b']['legacy'])

    def test_xml_case_b_full(self):
        xmldata = ET.fromstring(self.cases['b']['xml'])
        defaultdict = xml.to_dict(xmldata, True)
        self.assertEqual(defaultdict, self.cases['b']['full'])

    def test_xml_case_c(self):
        xmldata = ET.fromstring(self.cases['c']['xml'])
        defaultdict = xml.to_dict(xmldata)
        self.assertEqual(defaultdict, self.cases['c']['legacy'])

    def test_xml_case_c_legacy(self):
        xmldata = ET.fromstring(self.cases['c']['xml'])
        defaultdict = xml.to_dict(xmldata, False)
        self.assertEqual(defaultdict, self.cases['c']['legacy'])

    def test_xml_case_c_full(self):
        xmldata = ET.fromstring(self.cases['c']['xml'])
        defaultdict = xml.to_dict(xmldata, True)
        self.assertEqual(defaultdict, self.cases['c']['full'])

    def test_xml_case_d(self):
        xmldata = ET.fromstring(self.cases['d']['xml'])
        defaultdict = xml.to_dict(xmldata)
        self.assertEqual(defaultdict, self.cases['d']['legacy'])

    def test_xml_case_d_legacy(self):
        xmldata = ET.fromstring(self.cases['d']['xml'])
        defaultdict = xml.to_dict(xmldata, False)
        self.assertEqual(defaultdict, self.cases['d']['legacy'])

    def test_xml_case_d_full(self):
        xmldata = ET.fromstring(self.cases['d']['xml'])
        defaultdict = xml.to_dict(xmldata, True)
        self.assertEqual(defaultdict, self.cases['d']['full'])

    def test_xml_case_e(self):
        xmldata = ET.fromstring(self.cases['e']['xml'])
        defaultdict = xml.to_dict(xmldata)
        self.assertEqual(defaultdict, self.cases['e']['legacy'])

    def test_xml_case_e_legacy(self):
        xmldata = ET.fromstring(self.cases['e']['xml'])
        defaultdict = xml.to_dict(xmldata, False)
        self.assertEqual(defaultdict, self.cases['e']['legacy'])

    def test_xml_case_e_full(self):
        xmldata = ET.fromstring(self.cases['e']['xml'])
        defaultdict = xml.to_dict(xmldata, True)
        self.assertEqual(defaultdict, self.cases['e']['full'])

    def test_xml_case_f(self):
        xmldata = ET.fromstring(self.cases['f']['xml'])
        defaultdict = xml.to_dict(xmldata)
        self.assertEqual(defaultdict, self.cases['f']['legacy'])

    def test_xml_case_f_legacy(self):
        xmldata = ET.fromstring(self.cases['f']['xml'])
        defaultdict = xml.to_dict(xmldata, False)
        self.assertEqual(defaultdict, self.cases['f']['legacy'])

    def test_xml_case_f_full(self):
        xmldata = ET.fromstring(self.cases['f']['xml'])
        defaultdict = xml.to_dict(xmldata, True)
        self.assertEqual(defaultdict, self.cases['f']['full'])



# -*- coding: utf-8 -*-
'''
    :codeauthor: Rupesh Tare <rupesht@saltstack.com>
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import os.path

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.utils.crypt
import salt.modules.key as key


@skipIf(NO_MOCK, NO_MOCK_REASON)
class KeyTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.key
    '''
    def setup_loader_modules(self):
        return {key: {}}

    def test_finger(self):
        '''
        Test for finger
        '''
        with patch.object(os.path, 'join', return_value='A'):
            with patch.object(salt.utils.crypt,
                              'pem_finger', return_value='A'):
                with patch.dict(key.__opts__,
                        {'pki_dir': MagicMock(return_value='A'), 'hash_type': 'sha256'}):
                    self.assertEqual(key.finger(), 'A')

    def test_finger_master(self):
        '''
        Test for finger
        '''
        with patch.object(os.path, 'join', return_value='A'):
            with patch.object(salt.utils.crypt,
                              'pem_finger', return_value='A'):
                with patch.dict(key.__opts__,
                        {'pki_dir': 'A', 'hash_type': 'sha256'}):
                    self.assertEqual(key.finger_master(), 'A')



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Libs
import salt.modules.win_certutil as certutil

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase
from tests.support.mock import (
    MagicMock,
    patch
)


class CertUtilTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        return {certutil: {}}

    def test_get_serial(self):
        '''
            Test getting the serial number from a certificate
        '''
        expected = '180720d39cd2db3244ba037417241e90'
        mock = MagicMock(return_value=(
            'CertInfo\r\n'
            'Cert Serial Number: 180720d39cd2db3244ba037417241e90\r\n'
            '\r\n'
            'OtherStuff'))
        with patch.dict(certutil.__salt__, {'cmd.run': mock}):
            out = certutil.get_cert_serial('/path/to/cert.cer')
            mock.assert_called_once_with(
                'certutil.exe -silent -verify /path/to/cert.cer')
            self.assertEqual(expected, out)

    def test_get_serials(self):
        '''
            Test getting all the serial numbers from a store
        '''
        expected = ['180720d39cd2db3244ba037417241e90',
                    '1768ac4e5b72bf1d0df0df118b34b959']
        mock = MagicMock(return_value=(
            'CertInfo\r\n'
            '================ Certificate 0 ================\r\n'
            'Serial Number: 180720d39cd2db3244ba037417241e90\r\n'
            'OtherStuff\r\n'
            '\r\n'
            '================ Certificate 1 ================\r\n'
            'Serial Number: 1768ac4e5b72bf1d0df0df118b34b959\r\n'
            'OtherStuff'))
        with patch.dict(certutil.__salt__, {'cmd.run': mock}):
            out = certutil.get_stored_cert_serials('TrustedPublisher')
            mock.assert_called_once_with(
                'certutil.exe -store TrustedPublisher')
            self.assertEqual(expected, out)

    def test_add_store(self):
        '''
            Test adding a certificate to a specific store
        '''
        cmd_mock = MagicMock(return_value=(
            'CertInfo\r\n'
            '================ Certificate 0 ================\r\n'
            'Serial Number: 180720d39cd2db3244ba037417241e90\r\n'
            'OtherStuff'))
        cache_mock = MagicMock(return_value='/tmp/cert.cer')
        with patch.dict(certutil.__salt__, {'cmd.run': cmd_mock,
                                            'cp.cache_file': cache_mock}):
            certutil.add_store('salt://path/to/file', 'TrustedPublisher')
            cmd_mock.assert_called_once_with(
                'certutil.exe -addstore TrustedPublisher /tmp/cert.cer')
            cache_mock.assert_called_once_with('salt://path/to/file', 'base')

    def test_del_store(self):
        '''
            Test removing a certificate to a specific store
        '''
        with patch('salt.modules.win_certutil.get_cert_serial') as cert_serial_mock:
            cmd_mock = MagicMock(return_value=(
                'CertInfo\r\n'
                '================ Certificate 0 ================\r\n'
                'Serial Number: 180720d39cd2db3244ba037417241e90\r\n'
                'OtherStuff'))
            cache_mock = MagicMock(return_value='/tmp/cert.cer')
            cert_serial_mock.return_value = 'ABCDEF'
            with patch.dict(certutil.__salt__, {'cmd.run': cmd_mock,
                                                'cp.cache_file': cache_mock}):
                certutil.del_store('salt://path/to/file', 'TrustedPublisher')
                cmd_mock.assert_called_once_with(
                    'certutil.exe -delstore TrustedPublisher ABCDEF')
                cache_mock.assert_called_once_with('salt://path/to/file', 'base')



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.case import SyndicCase


class TestSyndic(SyndicCase):
    '''
    Validate the syndic interface by testing the test module
    '''
    def test_ping(self):
        '''
        test.ping
        '''
        self.assertTrue(self.run_function('test.ping'))

    def test_fib(self):
        '''
        test.fib
        '''
        self.assertEqual(
                self.run_function(
                    'test.fib',
                    ['20'],
                    )[0],
                6765
                )



# -*- coding: utf-8 -*-
# Import Python libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    MagicMock,
    NO_MOCK,
    NO_MOCK_REASON,
    patch)

# Import Salt Libs
import salt.states.openvswitch_port as openvswitch_port


@skipIf(NO_MOCK, NO_MOCK_REASON)
class OpenvswitchPortTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.states.openvswitch_port
    '''
    def setup_loader_modules(self):
        return {openvswitch_port: {'__opts__': {'test': False}}}

    # 'present' function tests: 1

    def test_present(self):
        '''
        Test to verify that the named port exists on bridge, eventually creates it.
        '''
        name = 'salt'
        bridge = 'br-salt'

        ret = {'name': name,
               'result': None,
               'comment': '',
               'changes': {}}

        mock = MagicMock(return_value=True)
        mock_l = MagicMock(return_value=['salt'])
        mock_n = MagicMock(return_value=[])

        with patch.dict(openvswitch_port.__salt__, {'openvswitch.bridge_exists': mock,
                                                    'openvswitch.port_list': mock_l
                                                    }):
            comt = 'Port salt already exists.'
            ret.update({'comment': comt, 'result': True})
            self.assertDictEqual(openvswitch_port.present(name, bridge), ret)

        with patch.dict(openvswitch_port.__salt__, {'openvswitch.bridge_exists': mock,
                                                    'openvswitch.port_list': mock_n,
                                                    'openvswitch.port_add': mock
                                                    }):
            comt = 'Port salt created on bridge br-salt.'
            ret.update({'comment': comt, 'result': True, 'changes':
                {'salt':
                     {'new': 'Created port salt on bridge br-salt.',
                      'old': 'No port named salt present.',
                      },
                 }
                        })
            self.assertDictEqual(openvswitch_port.present(name, bridge), ret)
        with patch.dict(openvswitch_port.__salt__, {'openvswitch.bridge_exists': mock,
                                                    'openvswitch.port_list': mock_n,
                                                    'openvswitch.port_add': mock,
                                                    'openvswitch.interface_get_options': mock_n,
                                                    'openvswitch.interface_get_type': MagicMock(return_value=''),
                                                    'openvswitch.port_create_gre': mock,
                                                    'dig.check_ip': mock,
                                                    }):
            comt = 'Port salt created on bridge br-salt.'
            self.maxDiff = None
            ret.update({'result': True,
                        'comment': 'Created GRE tunnel interface salt with remote ip 10.0.0.1  and key 1 on bridge br-salt.',
                        'changes':
                            {'salt':
                                {
                                    'new': 'Created GRE tunnel interface salt with remote ip 10.0.0.1 and key 1 on bridge br-salt.',
                                    'old': 'No GRE tunnel interface salt with remote ip 10.0.0.1 and key 1 on bridge br-salt present.',
                                },
                            }
                        })
            self.assertDictEqual(openvswitch_port.present(name, bridge, tunnel_type="gre", id=1, remote="10.0.0.1"), ret)



# -*- coding: utf-8 -*-
'''
tests.unit.returners.test_highstate_return
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Unit tests for the Highstate Returner Cache.
'''

# Import Python libs
from __future__ import absolute_import
import json
import logging
import os

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import TestCase

# Import Salt libs
import salt.utils.files
import salt.returners.highstate_return as highstate

log = logging.getLogger(__name__)


class HighstateReturnerTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Tests for the highstate_return returner
    '''
    output_file = os.path.join(RUNTIME_VARS.TMP, 'highstate_return')

    def tearDown(self):
        os.unlink(self.output_file)

    def setup_loader_modules(self):
        return {highstate: {'__opts__': {
            'highstate.report_everything': True,
            'highstate.report_format': 'json',
            'highstate.report_delivery': 'file',
            'highstate.file_output': self.output_file,
        }}}

    def test_pipe_in_name(self):
        ret = {
            'fun_args': ['test'],
            'jid': '20180308201402941603',
            'return': {
                'cmd_|-test_|-echo hi | grep h\n_|-run': {
                    'comment': 'Command "echo hi | grep h\n" run',
                    'name': 'echo hi | grep h\n',
                    'start_time': '20:14:03.053612',
                    'result': True,
                    'duration': 75.198,
                    '__run_num__': 0,
                    '__sls__': u'test',
                    'changes': {
                        'pid': 1429,
                        'retcode': 0,
                        'stderr': '',
                        'stdout': 'hi',
                    },
                    '__id__': 'test',
                }
            },
            'retcode': 0,
            'success': True,
            'fun': 'state.apply',
            'id': 'salt',
            'out': 'highstate',
        }
        expected = [
            {
                "stats": [
                    {"total": 1},
                    {"failed": 0, "__style__": "failed"},
                    {"unchanged": 0, "__style__": "unchanged"},
                    {"changed": 1, "__style__": "changed"}, {"duration": 75.198},
                ],
            },
            {
                "job": [
                    {"function": "state.apply"},
                    {"arguments": ["test"]},
                    {"jid": "20180308201402941603"},
                    {"success": True},
                    {"retcode": 0}
                ],
            },
            {
                "states": [
                    {
                        "test": [
                            {"function": "cmd.run"},
                            {"name": "echo hi | grep h\n"},
                            {"result": True},
                            {"duration": 75.198},
                            {"comment": "Command \"echo hi | grep h\n\" run"},
                            {"changes": [
                                {"pid": 1429},
                                {"retcode": 0},
                                {"stderr": ""},
                                {"stdout": "hi"}
                            ]},
                            {"started": "20:14:03.053612"}
                        ],
                        "__style__": "changed"
                    }
                ]
            }
        ]
        highstate.returner(ret)
        with salt.utils.files.fopen(self.output_file) as fh_:
            self.assertEqual(json.load(fh_), expected)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''
# Import Python libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch)

# Import Salt Libs
import salt.states.nexus as nexus


@skipIf(NO_MOCK, NO_MOCK_REASON)
class nexusTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.states.nexus
    '''
    def setup_loader_modules(self):
        return {nexus: {}}

    # 'downloaded' function tests: 1

    def test_downloaded(self):
        '''
        Test to ensures that the artifact from nexus exists at
        given location.
        '''
        name = 'jboss'
        arti_url = 'http://nexus.example.com/repository'
        artifact = {'nexus_url': arti_url, 'artifact_id': 'module',
                    'repository': 'libs-releases', 'packaging': 'jar',
                    'group_id': 'com.company.module', 'classifier': 'sources',
                    'version': '1.0'}

        ret = {'name': name,
               'result': False,
               'changes': {},
               'comment': ''}

        mck = MagicMock(return_value={'status': False, 'changes': {},
                                      'comment': ''})
        with patch.dict(nexus.__salt__, {'nexus.get_release': mck}):
            self.assertDictEqual(nexus.downloaded(name, artifact), ret)

        with patch.object(nexus, '__fetch_from_nexus',
                          MagicMock(side_effect=Exception('error'))):
            ret = nexus.downloaded(name, artifact)
            self.assertEqual(ret['result'], False)
            self.assertEqual(ret['comment'], 'error')



# -*- coding: utf-8 -*-
'''
    :codeauthor: Erik Johnson <erik@saltstack.com>
'''

# Import Python libs
from __future__ import absolute_import
import logging
import os

# Import Salt Testing Libs
from tests.support.helpers import with_tempdir
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    Mock,
    MagicMock,
    patch,
    DEFAULT,
    NO_MOCK,
    NO_MOCK_REASON,
)

# Import Salt Libs
import salt.states.git as git_state  # Don't potentially shadow GitPython

log = logging.getLogger(__name__)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class GitTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.states.git
    '''
    def setup_loader_modules(self):
        return {
            git_state: {
                '__env__': 'base',
                '__opts__': {'test': False},
                '__salt__': {},
            }
        }

    @with_tempdir()
    def test_latest_no_diff_for_bare_repo(self, target):
        '''
        This test ensures that we don't attempt to diff when cloning a repo
        using either bare=True or mirror=True.
        '''
        name = 'https://foo.com/bar/baz.git'
        gitdir = os.path.join(target, 'refs')
        isdir_mock = MagicMock(
            side_effect=lambda path: DEFAULT if path != gitdir else True)

        branches = ['foo', 'bar', 'baz']
        tags = ['v1.1.0', 'v.1.1.1', 'v1.2.0']
        local_head = 'b9ef06ab6b7524eb7c27d740dbbd5109c6d75ee4'
        remote_head = 'eef672c1ec9b8e613905dbcd22a4612e31162807'

        git_diff = Mock()
        dunder_salt = {
            'git.current_branch': MagicMock(return_value=branches[0]),
            'git.diff': git_diff,
            'git.fetch': MagicMock(return_value={}),
            'git.is_worktree': MagicMock(return_value=False),
            'git.list_branches': MagicMock(return_value=branches),
            'git.list_tags': MagicMock(return_value=tags),
            'git.remote_refs': MagicMock(return_value={'HEAD': remote_head}),
            'git.remotes': MagicMock(return_value={
                'origin': {'fetch': name, 'push': name},
            }),
            'git.rev_parse': MagicMock(side_effect=git_state.CommandExecutionError()),
            'git.revision': MagicMock(return_value=local_head),
            'git.version': MagicMock(return_value='1.8.3.1'),
        }
        with patch('os.path.isdir', isdir_mock), \
                patch.dict(git_state.__salt__, dunder_salt):
            result = git_state.latest(
                name=name,
                target=target,
                mirror=True,  # mirror=True implies bare=True
            )
            assert result['result'] is True, result
            git_diff.assert_not_called()



# -*- coding: utf-8 -*-
'''
Tests for the rabbitmq state
'''
# Import python libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.helpers import skip_if_not_root
from tests.support.mixins import SaltReturnAssertsMixin


@skip_if_not_root
class RabbitUserTestCase(ModuleCase, SaltReturnAssertsMixin):
    '''
    Validate the rabbitmq user states.
    '''
    def setUp(self):
        super(RabbitUserTestCase, self).setUp()
        rabbit_installed = self.run_function('cmd.has_exec', ['rabbitmqctl'])

        if not rabbit_installed:
            self.skipTest('rabbitmq-server not installed')

    def test_present(self):
        '''
        rabbitmq_user.present null_name
        '''
        ret = self.run_state(
            'rabbitmq_user.present', name='null_name', test=True
        )
        self.assertSaltFalseReturn(ret)
        self.assertInSaltComment('User \'null_name\' is set to be created', ret)

    def absent(self):
        '''
        rabbitmq_user.absent null_name
        '''
        ret = self.run_state(
            'rabbitmq_user.absent', name='null_name', test=True
        )
        self.assertSaltFalseReturn(ret)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import random

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.helpers import flaky
from tests.support.unit import skipIf

# Import Salt libs
from salt.ext import six
import salt.utils.platform


class StatusModuleTest(ModuleCase):
    '''
    Test the status module
    '''
    @skipIf(salt.utils.platform.is_windows(), 'minion is windows')
    @flaky
    def test_status_pid(self):
        '''
        status.pid
        '''
        status_pid = self.run_function('status.pid', ['salt'])
        grab_pids = status_pid.split()[:10]
        random_pid = random.choice(grab_pids)
        grep_salt = self.run_function('cmd.run', ['ps aux | grep salt'])
        self.assertIn(random_pid, grep_salt)

    @skipIf(not salt.utils.platform.is_windows(), 'windows only test')
    def test_status_cpuload(self):
        '''
        status.cpuload
        '''
        ret = self.run_function('status.cpuload')
        self.assertTrue(isinstance(ret, float))

    @skipIf(not salt.utils.platform.is_windows(), 'windows only test')
    def test_status_saltmem(self):
        '''
        status.saltmem
        '''
        ret = self.run_function('status.saltmem')
        self.assertTrue(isinstance(ret, int))

    def test_status_diskusage(self):
        '''
        status.diskusage
        '''
        ret = self.run_function('status.diskusage')
        if salt.utils.platform.is_darwin():
            self.assertIn('not yet supported on this platform', ret)
        elif salt.utils.platform.is_windows():
            self.assertTrue(isinstance(ret['percent'], float))
        else:
            self.assertIn('total', str(ret))
            self.assertIn('available', str(ret))

    def test_status_procs(self):
        '''
        status.procs
        '''
        ret = self.run_function('status.procs')
        for x, y in six.iteritems(ret):
            self.assertIn('cmd', y)

    def test_status_uptime(self):
        '''
        status.uptime
        '''
        ret = self.run_function('status.uptime')

        if salt.utils.platform.is_windows():
            self.assertTrue(isinstance(ret, float))
        else:
            self.assertTrue(isinstance(ret['days'], int))



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import tempfile

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.modules.debian_ip as debian_ip
import salt.utils.platform

# Import third party libs
import jinja2.exceptions


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(salt.utils.platform.is_windows(), 'Do not run these tests on Windows')
class DebianIpTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.debian_ip
    '''
    def setup_loader_modules(self):
        return {debian_ip: {}}

    # 'build_bond' function tests: 3

    def test_build_bond(self):
        '''
        Test if it create a bond script in /etc/modprobe.d with the passed
        settings and load the bonding kernel module.
        '''
        with patch('salt.modules.debian_ip._parse_settings_bond',
                   MagicMock(return_value={})), \
                           patch('salt.modules.debian_ip._write_file',
                                 MagicMock(return_value=True)):
            mock = MagicMock(return_value=1)
            with patch.dict(debian_ip.__grains__, {'osrelease': mock}):
                mock = MagicMock(return_value=True)
                with patch.dict(debian_ip.__salt__, {'kmod.load': mock,
                                                     'pkg.install': mock}):
                    self.assertEqual(debian_ip.build_bond('bond0'), '')

    def test_build_bond_exception(self):
        '''
        Test if it create a bond script in /etc/modprobe.d with the passed
        settings and load the bonding kernel module.
        '''
        with patch('salt.modules.debian_ip._parse_settings_bond',
                   MagicMock(return_value={})):
            mock = MagicMock(return_value=1)
            with patch.dict(debian_ip.__grains__, {'osrelease': mock}):
                mock = MagicMock(side_effect=
                                 jinja2.exceptions.TemplateNotFound('error'))
                with patch.object(jinja2.Environment, 'get_template', mock):
                    self.assertEqual(debian_ip.build_bond('bond0'), '')

    def test_build_bond_data(self):
        '''
        Test if it create a bond script in /etc/modprobe.d with the passed
        settings and load the bonding kernel module.
        '''
        with patch('salt.modules.debian_ip._parse_settings_bond',
                   MagicMock(return_value={})), \
                           patch('salt.modules.debian_ip._read_temp',
                                 MagicMock(return_value=True)):
            mock = MagicMock(return_value=1)
            with patch.dict(debian_ip.__grains__, {'osrelease': mock}):
                self.assertTrue(debian_ip.build_bond('bond0', test='True'))

    # 'build_routes' function tests: 2

    def test_build_routes(self):
        '''
        Test if it add route scripts for a network interface using up commands.
        '''
        with patch('salt.modules.debian_ip._parse_routes',
                   MagicMock(return_value={'routes': []})), \
                           patch('salt.modules.debian_ip._write_file_routes',
                                 MagicMock(return_value=True)), \
                                         patch('salt.modules.debian_ip._read_file',
                                               MagicMock(return_value='salt')):
            self.assertEqual(debian_ip.build_routes('eth0'), 'saltsalt')

    def test_build_routes_exception(self):
        '''
        Test if it add route scripts for a network interface using up commands.
        '''
        with patch('salt.modules.debian_ip._parse_routes',
                   MagicMock(return_value={'routes': []})):
            self.assertTrue(debian_ip.build_routes('eth0', test='True'))

            mock = MagicMock(side_effect=jinja2.exceptions.TemplateNotFound('err'))
            with patch.object(jinja2.Environment, 'get_template', mock):
                self.assertEqual(debian_ip.build_routes('eth0'), '')

    # 'down' function tests: 1

    def test_down(self):
        '''
        Test if it shutdown a network interface
        '''
        self.assertEqual(debian_ip.down('eth0', 'slave'), None)

        mock = MagicMock(return_value='Salt')
        with patch.dict(debian_ip.__salt__, {'cmd.run': mock}):
            self.assertEqual(debian_ip.down('eth0', 'eth'), 'Salt')

    # 'get_bond' function tests: 1

    def test_get_bond(self):
        '''
        Test if it return the content of a bond script
        '''
        self.assertEqual(debian_ip.get_bond('bond0'), '')

    # 'get_interface' function tests: 1

    def test_get_interface(self):
        '''
        Test if it return the contents of an interface script
        '''
        with patch.object(debian_ip, '_parse_interfaces',
                          MagicMock(return_value={})):
            self.assertListEqual(debian_ip.get_interface('eth0'), [])

        mock_ret = {'lo': {'enabled': True, 'data':
                           {'inet': {'addrfam': 'inet', 'proto': 'loopback'}}}}
        with patch.object(debian_ip, '_parse_interfaces',
                          MagicMock(return_value=mock_ret)):
            self.assertListEqual(debian_ip.get_interface('lo'),
                                 ['auto lo\n',
                                  'iface lo inet loopback\n',
                                  '\n'])

            mock = MagicMock(side_effect=jinja2.exceptions.TemplateNotFound
                             ('error'))
            with patch.object(jinja2.Environment, 'get_template', mock):
                self.assertEqual(debian_ip.get_interface('lo'), '')

    # 'build_interface' function tests: 1

    def test_build_interface(self):
        '''
        Test if it builds an interface script for a network interface.
        '''
        with patch('salt.modules.debian_ip._write_file_ifaces',
                   MagicMock(return_value='salt')):
            self.assertEqual(debian_ip.build_interface('eth0', 'eth', 'enabled'),
                             ['s\n', 'a\n', 'l\n', 't\n'])

            self.assertTrue(debian_ip.build_interface('eth0', 'eth', 'enabled',
                                                      test='True'))

            with patch.object(debian_ip, '_parse_settings_eth',
                              MagicMock(return_value={'routes': []})):
                self.assertRaises(AttributeError, debian_ip.build_interface,
                                  'eth0', 'bridge', 'enabled')

                self.assertRaises(AttributeError, debian_ip.build_interface,
                                  'eth0', 'slave', 'enabled')

                self.assertRaises(AttributeError, debian_ip.build_interface,
                                  'eth0', 'bond', 'enabled')

            self.assertTrue(debian_ip.build_interface('eth0', 'eth', 'enabled',
                                                      test='True'))

        interfaces = [
                # IPv4-only interface; single address
                {'iface_name': 'eth1', 'iface_type': 'eth', 'enabled': True,
                    'settings': {
                        'proto': 'static',
                        'ipaddr': '192.168.4.9',
                        'netmask': '255.255.255.0',
                        'gateway': '192.168.4.1',
                        'enable_ipv6': False,
                        'noifupdown': True,
                        },
                    'return': [
                        'auto eth1\n',
                        'iface eth1 inet static\n',
                        '    address 192.168.4.9\n',
                        '    netmask 255.255.255.0\n',
                        '    gateway 192.168.4.1\n',
                        '\n']},
                # IPv6-only; single address
                {'iface_name': 'eth2', 'iface_type': 'eth', 'enabled': True,
                    'settings': {
                        'proto': 'static',
                        'ipv6proto': 'static',
                        'ipv6ipaddr': '2001:db8:dead:beef::3',
                        'ipv6netmask': '64',
                        'ipv6gateway': '2001:db8:dead:beef::1',
                        'enable_ipv6': True,
                        'noifupdown': True,
                        },
                    'return': [
                        'auto eth2\n',
                        'iface eth2 inet6 static\n',
                        '    address 2001:db8:dead:beef::3\n',
                        '    netmask 64\n',
                        '    gateway 2001:db8:dead:beef::1\n',
                        '\n']},
                # IPv4 and IPv6; shared/overridden settings
                {'iface_name': 'eth3', 'iface_type': 'eth', 'enabled': True,
                    'settings': {
                        'proto': 'static',
                        'ipaddr': '192.168.4.9',
                        'netmask': '255.255.255.0',
                        'gateway': '192.168.4.1',
                        'ipv6proto': 'static',
                        'ipv6ipaddr': '2001:db8:dead:beef::3',
                        'ipv6netmask': '64',
                        'ipv6gateway': '2001:db8:dead:beef::1',
                        'ttl': '18',  # shared
                        'ipv6ttl': '15',  # overriden for v6
                        'mtu': '1480',  # shared
                        'enable_ipv6': True,
                        'noifupdown': True,
                        },
                    'return': [
                        'auto eth3\n',
                        'iface eth3 inet static\n',
                        '    address 192.168.4.9\n',
                        '    netmask 255.255.255.0\n',
                        '    gateway 192.168.4.1\n',
                        '    ttl 18\n',
                        '    mtu 1480\n',
                        'iface eth3 inet6 static\n',
                        '    address 2001:db8:dead:beef::3\n',
                        '    netmask 64\n',
                        '    gateway 2001:db8:dead:beef::1\n',
                        '    ttl 15\n',
                        '    mtu 1480\n',
                        '\n']},
                # Slave iface
                {'iface_name': 'eth4', 'iface_type': 'slave', 'enabled': True,
                    'settings': {
                        'master': 'bond0',
                        'noifupdown': True,
                        },
                    'return': [
                        'auto eth4\n',
                        'iface eth4 inet manual\n',
                        '    bond-master bond0\n',
                        '\n']},
                # Bond; with address IPv4 and IPv6 address; slaves as string
                {'iface_name': 'bond5', 'iface_type': 'bond', 'enabled': True,
                    'settings': {
                        'proto': 'static',
                        'ipaddr': '10.1.0.14',
                        'netmask': '255.255.255.0',
                        'gateway': '10.1.0.1',
                        'ipv6proto': 'static',
                        'ipv6ipaddr': '2001:db8:dead:c0::3',
                        'ipv6netmask': '64',
                        'ipv6gateway': '2001:db8:dead:c0::1',
                        'mode': '802.3ad',
                        'slaves': 'eth4 eth5',
                        'enable_ipv6': True,
                        'noifupdown': True,
                        },
                    'return': [
                        'auto bond5\n',
                        'iface bond5 inet static\n',
                        '    address 10.1.0.14\n',
                        '    netmask 255.255.255.0\n',
                        '    gateway 10.1.0.1\n',
                        '    bond-ad_select 0\n',
                        '    bond-downdelay 200\n',
                        '    bond-lacp_rate 0\n',
                        '    bond-miimon 100\n',
                        '    bond-mode 4\n',
                        '    bond-slaves eth4 eth5\n',
                        '    bond-updelay 0\n',
                        '    bond-use_carrier on\n',
                        'iface bond5 inet6 static\n',
                        '    address 2001:db8:dead:c0::3\n',
                        '    netmask 64\n',
                        '    gateway 2001:db8:dead:c0::1\n',
                             # TODO: I suspect there should be more here.
                        '\n']},
                # Bond; with address IPv4 and IPv6 address; slaves as list
                {'iface_name': 'bond6', 'iface_type': 'bond', 'enabled': True,
                    'settings': {
                        'proto': 'static',
                        'ipaddr': '10.1.0.14',
                        'netmask': '255.255.255.0',
                        'gateway': '10.1.0.1',
                        'ipv6proto': 'static',
                        'ipv6ipaddr': '2001:db8:dead:c0::3',
                        'ipv6netmask': '64',
                        'ipv6gateway': '2001:db8:dead:c0::1',
                        'mode': '802.3ad',
                        # TODO: Need to add this support
                        #'slaves': ['eth4', 'eth5'],
                        'slaves': 'eth4 eth5',
                        'enable_ipv6': True,
                        'noifupdown': True,
                        },
                    'return': [
                        'auto bond6\n',
                        'iface bond6 inet static\n',
                        '    address 10.1.0.14\n',
                        '    netmask 255.255.255.0\n',
                        '    gateway 10.1.0.1\n',
                        '    bond-ad_select 0\n',
                        '    bond-downdelay 200\n',
                        '    bond-lacp_rate 0\n',
                        '    bond-miimon 100\n',
                        '    bond-mode 4\n',
                        '    bond-slaves eth4 eth5\n',
                        '    bond-updelay 0\n',
                        '    bond-use_carrier on\n',
                        'iface bond6 inet6 static\n',
                        '    address 2001:db8:dead:c0::3\n',
                        '    netmask 64\n',
                        '    gateway 2001:db8:dead:c0::1\n',
                             # TODO: I suspect there should be more here.
                        '\n']},
                # Bond VLAN; with IPv4 address
                {'iface_name': 'bond1.7', 'iface_type': 'vlan', 'enabled': True,
                    'settings': {
                        'proto': 'static',
                        'ipaddr': '10.7.0.8',
                        'netmask': '255.255.255.0',
                        'gateway': '10.7.0.1',
                        'slaves': 'eth6 eth7',
                        'mode': '802.3ad',
                        'enable_ipv6': False,
                        'noifupdown': True,
                        },
                    'return': [
                        'auto bond1.7\n',
                        'iface bond1.7 inet static\n',
                        '    vlan-raw-device bond1\n',
                        '    address 10.7.0.8\n',
                        '    netmask 255.255.255.0\n',
                        '    gateway 10.7.0.1\n',
                        '    mode 802.3ad\n',
                        '\n']},
                # Bond; without address
                {'iface_name': 'bond1.8', 'iface_type': 'vlan', 'enabled': True,
                    'settings': {
                        'proto': 'static',
                        'slaves': 'eth6 eth7',
                        'mode': '802.3ad',
                        'enable_ipv6': False,
                        'noifupdown': True,
                        },
                    'return': [
                        'auto bond1.8\n',
                        'iface bond1.8 inet static\n',
                        '    vlan-raw-device bond1\n',
                        '    mode 802.3ad\n',
                        '\n']},
                # DNS NS as list
                {'iface_name': 'eth9', 'iface_type': 'eth', 'enabled': True,
                    'settings': {
                        'proto': 'static',
                        'ipaddr': '192.168.4.9',
                        'netmask': '255.255.255.0',
                        'gateway': '192.168.4.1',
                        'enable_ipv6': False,
                        'noifupdown': True,
                        'dns': ['8.8.8.8', '8.8.4.4'],
                        },
                    'return': [
                        'auto eth9\n',
                        'iface eth9 inet static\n',
                        '    address 192.168.4.9\n',
                        '    netmask 255.255.255.0\n',
                        '    gateway 192.168.4.1\n',
                        '    dns-nameservers 8.8.8.8 8.8.4.4\n',
                        '\n']},
                # DNS NS as string
                {'iface_name': 'eth10', 'iface_type': 'eth', 'enabled': True,
                    'settings': {
                        'proto': 'static',
                        'ipaddr': '192.168.4.9',
                        'netmask': '255.255.255.0',
                        'gateway': '192.168.4.1',
                        'enable_ipv6': False,
                        'noifupdown': True,
                        'dns': '8.8.8.8 8.8.4.4',
                        },
                    'return': [
                        'auto eth10\n',
                        'iface eth10 inet static\n',
                        '    address 192.168.4.9\n',
                        '    netmask 255.255.255.0\n',
                        '    gateway 192.168.4.1\n',
                        '    dns-nameservers 8.8.8.8 8.8.4.4\n',
                        '\n']},
                # Loopback; with IPv4 and IPv6 address
                {'iface_name': 'lo11', 'iface_type': 'eth', 'enabled': True,
                    'settings': {
                        'proto': 'loopback',
                        'ipaddr': '192.168.4.9',
                        'netmask': '255.255.255.0',
                        'gateway': '192.168.4.1',
                        'ipv6ipaddr': 'fc00::1',
                        'ipv6netmask': '128',
                        'ipv6_autoconf': False,
                        'enable_ipv6': True,
                        'noifupdown': True,
                        },
                    'return': [
                        'auto lo11\n',
                        'iface lo11 inet loopback\n',
                        '    address 192.168.4.9\n',
                        '    netmask 255.255.255.0\n',
                        '    gateway 192.168.4.1\n',
                        'iface lo11 inet6 loopback\n',
                        '    address fc00::1\n',
                        '    netmask 128\n',
                        '\n']},
                # Loopback; without address
                {'iface_name': 'lo12', 'iface_type': 'eth', 'enabled': True,
                    'settings': {
                        'proto': 'loopback',
                        'enable_ipv6': False,
                        'noifupdown': True,
                        },
                    'return': [
                        'auto lo12\n',
                        'iface lo12 inet loopback\n',
                        '\n']},
                ]

        with tempfile.NamedTemporaryFile(mode='r', delete=True) as tfile:
            with patch('salt.modules.debian_ip._DEB_NETWORK_FILE', str(tfile.name)):
                for iface in interfaces:
                    # Skip tests that require __salt__['pkg.install']()
                    if iface['iface_type'] not in ['bridge', 'pppoe', 'vlan']:
                        self.assertListEqual(debian_ip.build_interface(
                                                    iface=iface['iface_name'],
                                                    iface_type=iface['iface_type'],
                                                    enabled=iface['enabled'],
                                                    interface_file=tfile.name,
                                                    **iface['settings']),
                                             iface['return'])

    # 'up' function tests: 1

    def test_up(self):
        '''
        Test if it start up a network interface
        '''
        self.assertEqual(debian_ip.down('eth0', 'slave'), None)

        mock = MagicMock(return_value='Salt')
        with patch.dict(debian_ip.__salt__, {'cmd.run': mock}):
            self.assertEqual(debian_ip.up('eth0', 'eth'), 'Salt')

    # 'get_network_settings' function tests: 1

    def test_get_network_settings(self):
        '''
        Test if it return the contents of the global network script.
        '''
        with patch.dict(debian_ip.__grains__, {'osfullname': 'Ubuntu',
                                               'osrelease': '14'}), \
                patch('salt.modules.debian_ip._parse_hostname',
                      MagicMock(return_value='SaltStack')), \
                        patch('salt.modules.debian_ip._parse_domainname',
                              MagicMock(return_value='saltstack.com')), \
                                      patch('salt.modules.debian_ip._parse_searchdomain',
                                              MagicMock(return_value='test.saltstack.com')):
            mock_avai = MagicMock(return_value=True)
            with patch.dict(debian_ip.__salt__, {'service.available': mock_avai,
                                                 'service.status': mock_avai}):
                self.assertEqual(debian_ip.get_network_settings(),
                                 [u'NETWORKING=yes\n',
                                  u'HOSTNAME=SaltStack\n',
                                  u'DOMAIN=saltstack.com\n',
                                  u'SEARCH=test.saltstack.com\n'])

                mock = MagicMock(side_effect=jinja2.exceptions.TemplateNotFound
                                 ('error'))
                with patch.object(jinja2.Environment, 'get_template', mock):
                    self.assertEqual(debian_ip.get_network_settings(), '')

    # 'get_routes' function tests: 1

    def test_get_routes(self):
        '''
        Test if it return the routes for the interface
        '''
        with patch('salt.modules.debian_ip._read_file', MagicMock(return_value='salt')):
            self.assertEqual(debian_ip.get_routes('eth0'), 'saltsalt')

    # 'apply_network_settings' function tests: 1

    def test_apply_network_settings(self):
        '''
        Test if it apply global network configuration.
        '''
        mock = MagicMock(return_value=True)
        with patch.dict(debian_ip.__salt__, {'network.mod_hostname': mock,
                                             'service.stop': mock,
                                             'service.start': mock}):
            self.assertEqual(debian_ip.apply_network_settings(), True)

    # 'build_network_settings' function tests: 1

    def test_build_network_settings(self):
        '''
        Test if it build the global network script.
        '''
        with patch('salt.modules.debian_ip._parse_network_settings',
                   MagicMock(return_value={'networking': 'yes',
                                           'hostname': 'Salt.saltstack.com',
                                           'domainname': 'saltstack.com',
                                           'search': 'test.saltstack.com'})), \
                patch('salt.modules.debian_ip._write_file_network',
                      MagicMock(return_value=True)):
            with patch.dict(debian_ip.__grains__, {'osfullname': 'Ubuntu',
                                                   'osrelease': '14'}):
                mock = MagicMock(return_value=True)
                with patch.dict(debian_ip.__salt__, {'service.available': mock,
                                                     'service.disable': mock,
                                                     'service.enable': mock}):
                    self.assertEqual(debian_ip.build_network_settings(),
                                     ['NETWORKING=yes\n',
                                      'HOSTNAME=Salt\n',
                                      'DOMAIN=saltstack.com\n',
                                      'SEARCH=test.saltstack.com\n'])

                    mock = MagicMock(side_effect=jinja2.exceptions.TemplateNotFound
                                     ('error'))
                    with patch.object(jinja2.Environment, 'get_template', mock):
                        self.assertEqual(debian_ip.build_network_settings(), '')

            with patch.dict(debian_ip.__grains__, {'osfullname': 'Ubuntu',
                                                   'osrelease': '10'}):
                mock = MagicMock(return_value=True)
                with patch.dict(debian_ip.__salt__, {'service.available': mock,
                                                     'service.disable': mock,
                                                     'service.enable': mock}):
                    mock = MagicMock(side_effect=jinja2.exceptions.TemplateNotFound
                                     ('error'))
                    with patch.object(jinja2.Environment, 'get_template', mock):
                        self.assertEqual(debian_ip.build_network_settings(), '')

                    with patch.object(debian_ip, '_read_temp',
                                      MagicMock(return_value=True)):
                        self.assertTrue(debian_ip.build_network_settings
                                        (test='True'))



# -*- coding: utf-8 -*-
'''
Testing ini_manage exec module.
'''
# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import tempfile

# Import Salt Testing libs
from tests.support.unit import TestCase

# Import Salt libs
import salt.utils.files
import salt.utils.stringutils
import salt.modules.ini_manage as ini


class IniManageTestCase(TestCase):
    '''
    Testing ini_manage exec module.
    '''

    TEST_FILE_CONTENT = os.linesep.join([
        '# Comment on the first line',
        '',
        '# First main option',
        'option1=main1',
        '',
        '# Second main option',
        'option2=main2',
        '',
        '',
        '[main]',
        '# Another comment',
        'test1=value 1',
        '',
        'test2=value 2',
        '',
        '[SectionB]',
        'test1=value 1B',
        '',
        '# Blank line should be above',
        'test3 = value 3B',
        '',
        '[SectionC]',
        '# The following option is empty',
        'empty_option='
    ])

    maxDiff = None

    def setUp(self):
        self.tfile = tempfile.NamedTemporaryFile(delete=False, mode='w+b')
        self.tfile.write(salt.utils.stringutils.to_bytes(self.TEST_FILE_CONTENT))
        self.tfile.close()

    def tearDown(self):
        os.remove(self.tfile.name)

    def test_get_option(self):
        '''
        Test get_option method.
        '''
        self.assertEqual(
            ini.get_option(self.tfile.name, 'main', 'test1'),
            'value 1')
        self.assertEqual(
            ini.get_option(self.tfile.name, 'main', 'test2'),
            'value 2')
        self.assertEqual(
            ini.get_option(self.tfile.name, 'SectionB', 'test1'),
            'value 1B')
        self.assertEqual(
            ini.get_option(self.tfile.name, 'SectionB', 'test3'),
            'value 3B')
        self.assertEqual(
            ini.get_option(self.tfile.name, 'SectionC', 'empty_option'),
            '')

    def test_get_section(self):
        '''
        Test get_section method.
        '''
        self.assertEqual(
            ini.get_section(self.tfile.name, 'SectionB'),
            {'test1': 'value 1B', 'test3': 'value 3B'})

    def test_remove_option(self):
        '''
        Test remove_option method.
        '''
        self.assertEqual(
            ini.remove_option(self.tfile.name, 'SectionB', 'test1'),
            'value 1B')
        self.assertIsNone(ini.get_option(self.tfile.name, 'SectionB', 'test1'))

    def test_remove_section(self):
        '''
        Test remove_section method.
        '''
        self.assertEqual(
            ini.remove_section(self.tfile.name, 'SectionB'),
            {'test1': 'value 1B', 'test3': 'value 3B'})
        self.assertEqual(ini.get_section(self.tfile.name, 'SectionB'), {})

    def test_get_ini(self):
        '''
        Test get_ini method.
        '''
        self.assertEqual(
            dict(ini.get_ini(self.tfile.name)), {
                'SectionC': {'empty_option': ''},
                'SectionB': {'test1': 'value 1B', 'test3': 'value 3B'},
                'main': {'test1': 'value 1', 'test2': 'value 2'},
                'option2': 'main2', 'option1': 'main1'})

    def test_set_option(self):
        '''
        Test set_option method.
        '''
        result = ini.set_option(self.tfile.name, {
            'SectionB': {
                'test3': 'new value 3B',
                'test_set_option': 'test_set_value'
            },
            'SectionD': {
                'test_set_option2': 'test_set_value1'
            }
        })
        self.assertEqual(result, {
            'SectionB': {'test3': {'after': 'new value 3B',
                                   'before': 'value 3B'},
                         'test_set_option': {'after': 'test_set_value',
                                             'before': None}},
            'SectionD': {'after': {'test_set_option2': 'test_set_value1'},
                         'before': None}})
        # Check existing option updated
        self.assertEqual(
            ini.get_option(self.tfile.name, 'SectionB', 'test3'),
            'new value 3B')
        # Check new section and option added
        self.assertEqual(
            ini.get_option(self.tfile.name, 'SectionD', 'test_set_option2'),
            'test_set_value1')

    def test_empty_value(self):
        '''
        Test empty value preserved after edit
        '''
        ini.set_option(self.tfile.name, {
            'SectionB': {'test3': 'new value 3B'},
        })
        with salt.utils.files.fopen(self.tfile.name, 'r') as fp_:
            file_content = salt.utils.stringutils.to_unicode(fp_.read())
        expected = '{0}{1}{0}'.format(os.linesep, 'empty_option = ')
        self.assertIn(expected, file_content, 'empty_option was not preserved')

    def test_empty_lines(self):
        '''
        Test empty lines preserved after edit
        '''
        ini.set_option(self.tfile.name, {
            'SectionB': {'test3': 'new value 3B'},
        })
        expected = os.linesep.join([
            '# Comment on the first line',
            '',
            '# First main option',
            'option1 = main1',
            '',
            '# Second main option',
            'option2 = main2',
            '',
            '[main]',
            '# Another comment',
            'test1 = value 1',
            '',
            'test2 = value 2',
            '',
            '[SectionB]',
            'test1 = value 1B',
            '',
            '# Blank line should be above',
            'test3 = new value 3B',
            '',
            '[SectionC]',
            '# The following option is empty',
            'empty_option = ',
            ''
        ])
        with salt.utils.files.fopen(self.tfile.name, 'r') as fp_:
            file_content = salt.utils.stringutils.to_unicode(fp_.read())
        self.assertEqual(expected, file_content)

    def test_empty_lines_multiple_edits(self):
        '''
        Test empty lines preserved after multiple edits
        '''
        ini.set_option(self.tfile.name, {
            'SectionB': {'test3': 'this value will be edited two times'},
        })
        self.test_empty_lines()



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import sys
import tempfile
import textwrap

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.helpers import (
    destructiveTest,
    skip_if_binaries_missing,
    skip_if_not_root,
    this_user,
)
from tests.support.paths import TMP
from tests.support.unit import skipIf

# Import salt libs
import salt.utils.path
import salt.utils.platform

# Import 3rd-party libs
from salt.ext import six

AVAILABLE_PYTHON_EXECUTABLE = salt.utils.path.which_bin([
    'python',
    'python2',
    'python2.6',
    'python2.7'

])


class CMDModuleTest(ModuleCase):
    '''
    Validate the cmd module
    '''
    def setUp(self):
        self.runas_usr = 'nobody'
        if salt.utils.platform.is_darwin():
            self.runas_usr = 'macsalttest'

    def tearDown(self):
        if self._testMethodName == 'test_runas':
            if salt.utils.platform.is_darwin():
                if self.runas_usr in self.run_function('user.info', [self.runas_usr]).values():
                    self.run_function('user.delete', [self.runas_usr], remove=True)

    def test_run(self):
        '''
        cmd.run
        '''
        shell = os.environ.get('SHELL')
        if shell is None:
            # Failed to get the SHELL var, don't run
            self.skipTest('Unable to get the SHELL environment variable')

        self.assertTrue(self.run_function('cmd.run', ['echo $SHELL']))
        self.assertEqual(
            self.run_function('cmd.run',
                              ['echo $SHELL',
                               'shell={0}'.format(shell)],
                              python_shell=True).rstrip(), shell)
        self.assertEqual(self.run_function('cmd.run',
                          ['ls / | grep etc'],
                          python_shell=True), 'etc')
        self.assertEqual(self.run_function('cmd.run',
                         ['echo {{grains.id}} | awk "{print $1}"'],
                         template='jinja',
                         python_shell=True), 'minion')
        self.assertEqual(self.run_function('cmd.run',
                         ['grep f'],
                         stdin='one\ntwo\nthree\nfour\nfive\n'), 'four\nfive')
        self.assertEqual(self.run_function('cmd.run',
                         ['echo "a=b" | sed -e s/=/:/g'],
                         python_shell=True), 'a:b')

    def test_stdout(self):
        '''
        cmd.run_stdout
        '''
        self.assertEqual(self.run_function('cmd.run_stdout',
                                           ['echo "cheese"']).rstrip(),
                         'cheese' if not salt.utils.platform.is_windows() else '"cheese"')

    def test_stderr(self):
        '''
        cmd.run_stderr
        '''
        if sys.platform.startswith(('freebsd', 'openbsd')):
            shell = '/bin/sh'
        else:
            shell = '/bin/bash'

        self.assertEqual(self.run_function('cmd.run_stderr',
                                           ['echo "cheese" 1>&2',
                                            'shell={0}'.format(shell)], python_shell=True
                                           ).rstrip(),
                         'cheese' if not salt.utils.platform.is_windows() else '"cheese"')

    def test_run_all(self):
        '''
        cmd.run_all
        '''
        if sys.platform.startswith(('freebsd', 'openbsd')):
            shell = '/bin/sh'
        else:
            shell = '/bin/bash'

        ret = self.run_function('cmd.run_all', ['echo "cheese" 1>&2',
                                                'shell={0}'.format(shell)], python_shell=True)
        self.assertTrue('pid' in ret)
        self.assertTrue('retcode' in ret)
        self.assertTrue('stdout' in ret)
        self.assertTrue('stderr' in ret)
        self.assertTrue(isinstance(ret.get('pid'), int))
        self.assertTrue(isinstance(ret.get('retcode'), int))
        self.assertTrue(isinstance(ret.get('stdout'), six.string_types))
        self.assertTrue(isinstance(ret.get('stderr'), six.string_types))
        self.assertEqual(ret.get('stderr').rstrip(), 'cheese' if not salt.utils.platform.is_windows() else '"cheese"')

    def test_retcode(self):
        '''
        cmd.retcode
        '''
        self.assertEqual(self.run_function('cmd.retcode', ['exit 0'], python_shell=True), 0)
        self.assertEqual(self.run_function('cmd.retcode', ['exit 1'], python_shell=True), 1)

    def test_blacklist_glob(self):
        '''
        cmd_blacklist_glob
        '''
        self.assertEqual(self.run_function('cmd.run',
                ['bad_command --foo']).rstrip(),
                'ERROR: The shell command "bad_command --foo" is not permitted')

    def test_script(self):
        '''
        cmd.script
        '''
        args = 'saltines crackers biscuits=yes'
        script = 'salt://script.py'
        ret = self.run_function('cmd.script', [script, args])
        self.assertEqual(ret['stdout'], args)

    def test_script_retcode(self):
        '''
        cmd.script_retcode
        '''
        script = 'salt://script.py'
        ret = self.run_function('cmd.script_retcode', [script])
        self.assertEqual(ret, 0)

    def test_script_cwd(self):
        '''
        cmd.script with cwd
        '''
        tmp_cwd = tempfile.mkdtemp(dir=TMP)
        args = 'saltines crackers biscuits=yes'
        script = 'salt://script.py'
        ret = self.run_function('cmd.script', [script, args], cwd=tmp_cwd)
        self.assertEqual(ret['stdout'], args)

    def test_script_cwd_with_space(self):
        '''
        cmd.script with cwd
        '''
        tmp_cwd = "{0}{1}test 2".format(tempfile.mkdtemp(dir=TMP), os.path.sep)
        os.mkdir(tmp_cwd)

        args = 'saltines crackers biscuits=yes'
        script = 'salt://script.py'
        ret = self.run_function('cmd.script', [script, args], cwd=tmp_cwd)
        self.assertEqual(ret['stdout'], args)

    @destructiveTest
    def test_tty(self):
        '''
        cmd.tty
        '''
        for tty in ('tty0', 'pts3'):
            if os.path.exists(os.path.join('/dev', tty)):
                ret = self.run_function('cmd.tty', [tty, 'apply salt liberally'])
                self.assertTrue('Success' in ret)

    @skip_if_binaries_missing(['which'])
    def test_which(self):
        '''
        cmd.which
        '''
        self.assertEqual(self.run_function('cmd.which', ['cat']).rstrip(),
                         self.run_function('cmd.run', ['which cat']).rstrip())

    @skip_if_binaries_missing(['which'])
    def test_which_bin(self):
        '''
        cmd.which_bin
        '''
        cmds = ['pip3', 'pip2', 'pip', 'pip-python']
        ret = self.run_function('cmd.which_bin', [cmds])
        self.assertTrue(os.path.split(ret)[1] in cmds)

    def test_has_exec(self):
        '''
        cmd.has_exec
        '''
        self.assertTrue(self.run_function('cmd.has_exec',
                                          [AVAILABLE_PYTHON_EXECUTABLE]))
        self.assertFalse(self.run_function('cmd.has_exec',
                                           ['alllfsdfnwieulrrh9123857ygf']))

    def test_exec_code(self):
        '''
        cmd.exec_code
        '''
        code = textwrap.dedent('''\
               import sys
               sys.stdout.write('cheese')''')
        self.assertEqual(self.run_function('cmd.exec_code',
                                           [AVAILABLE_PYTHON_EXECUTABLE,
                                            code]).rstrip(),
                         'cheese')

    def test_exec_code_with_single_arg(self):
        '''
        cmd.exec_code
        '''
        code = textwrap.dedent('''\
               import sys
               sys.stdout.write(sys.argv[1])''')
        arg = 'cheese'
        self.assertEqual(self.run_function('cmd.exec_code',
                                           [AVAILABLE_PYTHON_EXECUTABLE,
                                            code],
                                           args=arg).rstrip(),
                         arg)

    def test_exec_code_with_multiple_args(self):
        '''
        cmd.exec_code
        '''
        code = textwrap.dedent('''\
               import sys
               sys.stdout.write(sys.argv[1])''')
        arg = 'cheese'
        self.assertEqual(self.run_function('cmd.exec_code',
                                           [AVAILABLE_PYTHON_EXECUTABLE,
                                            code],
                                           args=[arg, 'test']).rstrip(),
                         arg)

    def test_quotes(self):
        '''
        cmd.run with quoted command
        '''
        cmd = '''echo 'SELECT * FROM foo WHERE bar="baz"' '''
        expected_result = 'SELECT * FROM foo WHERE bar="baz"'
        if salt.utils.platform.is_windows():
            expected_result = '\'SELECT * FROM foo WHERE bar="baz"\''
        result = self.run_function('cmd.run_stdout', [cmd]).strip()
        self.assertEqual(result, expected_result)

    @skip_if_not_root
    @skipIf(salt.utils.platform.is_windows, 'skip windows, requires password')
    def test_quotes_runas(self):
        '''
        cmd.run with quoted command
        '''
        cmd = '''echo 'SELECT * FROM foo WHERE bar="baz"' '''
        if salt.utils.platform.is_darwin():
            cmd = '''echo 'SELECT * FROM foo WHERE bar=\\"baz\\"' '''

        expected_result = 'SELECT * FROM foo WHERE bar="baz"'

        runas = this_user()

        result = self.run_function('cmd.run_stdout', [cmd],
                                   runas=runas).strip()
        self.assertEqual(result, expected_result)

    @skipIf(salt.utils.platform.is_windows(), 'minion is windows')
    @skip_if_not_root
    @destructiveTest
    def test_runas(self):
        '''
        Ensure that the env is the runas user's
        '''
        if salt.utils.platform.is_darwin():
            if self.runas_usr not in self.run_function('user.info', [self.runas_usr]).values():
                self.run_function('user.add', [self.runas_usr])

        out = self.run_function('cmd.run', ['env'], runas=self.runas_usr).splitlines()
        self.assertIn('USER={0}'.format(self.runas_usr), out)

    @skipIf(not salt.utils.path.which_bin('sleep'), 'sleep cmd not installed')
    def test_timeout(self):
        '''
        cmd.run trigger timeout
        '''
        out = self.run_function('cmd.run',
                                ['sleep 2 && echo hello'],
                                f_timeout=1,
                                python_shell=True)
        self.assertTrue('Timed out' in out)

    @skipIf(not salt.utils.path.which_bin('sleep'), 'sleep cmd not installed')
    def test_timeout_success(self):
        '''
        cmd.run sufficient timeout to succeed
        '''
        out = self.run_function('cmd.run',
                                ['sleep 1 && echo hello'],
                                f_timeout=2,
                                python_shell=True)
        self.assertEqual(out, 'hello')

    def test_hide_output(self):
        '''
        Test the hide_output argument
        '''
        ls_command = ['ls', '/'] \
            if not salt.utils.platform.is_windows() \
            else ['dir', 'c:\\']

        error_command = ['thiscommanddoesnotexist']

        # cmd.run
        out = self.run_function(
            'cmd.run',
            ls_command,
            hide_output=True)
        self.assertEqual(out, '')

        # cmd.shell
        out = self.run_function(
            'cmd.shell',
            ls_command,
            hide_output=True)
        self.assertEqual(out, '')

        # cmd.run_stdout
        out = self.run_function(
            'cmd.run_stdout',
            ls_command,
            hide_output=True)
        self.assertEqual(out, '')

        # cmd.run_stderr
        out = self.run_function(
            'cmd.shell',
            error_command,
            hide_output=True)
        self.assertEqual(out, '')

        # cmd.run_all (command should have produced stdout)
        out = self.run_function(
            'cmd.run_all',
            ls_command,
            hide_output=True)
        self.assertEqual(out['stdout'], '')
        self.assertEqual(out['stderr'], '')

        # cmd.run_all (command should have produced stderr)
        out = self.run_function(
            'cmd.run_all',
            error_command,
            hide_output=True)
        self.assertEqual(out['stdout'], '')
        self.assertEqual(out['stderr'], '')

    def test_cmd_run_whoami(self):
        '''
        test return of whoami
        '''
        cmd = self.run_function('cmd.run', ['whoami'])
        if salt.utils.platform.is_windows():
            self.assertIn('administrator', cmd)
        else:
            self.assertEqual('root', cmd)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Amel Ajdinovic <amel@stackpointcloud.com>`
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals
import os

# Import Salt Testing Libs
from tests.support.case import ShellCase
from tests.support.paths import FILES
from tests.support.unit import skipIf
from tests.support.helpers import expensiveTest, generate_random_name

# Import Salt Libs
from salt.config import cloud_providers_config

# Import Third-Party Libs
try:
    from oneandone.client import OneAndOneService  # pylint: disable=unused-import
    HAS_ONEANDONE = True
except ImportError:
    HAS_ONEANDONE = False

# Create the cloud instance name to be used throughout the tests
INSTANCE_NAME = generate_random_name('CLOUD-TEST-')
PROVIDER_NAME = 'oneandone'
DRIVER_NAME = 'oneandone'


@skipIf(HAS_ONEANDONE is False, 'salt-cloud requires >= 1and1 1.2.0')
class OneAndOneTest(ShellCase):
    '''
    Integration tests for the 1and1 cloud provider
    '''

    @expensiveTest
    def setUp(self):
        '''
        Sets up the test requirements
        '''
        super(OneAndOneTest, self).setUp()

        # check if appropriate cloud provider and profile files are present
        profile_str = 'oneandone-config'
        providers = self.run_cloud('--list-providers')
        if profile_str + ':' not in providers:
            self.skipTest(
                'Configuration file for {0} was not found. Check {0}.conf '
                'files in tests/integration/files/conf/cloud.*.d/ to run '
                'these tests.'.format(PROVIDER_NAME)
            )

        # check if api_token present
        config = cloud_providers_config(
            os.path.join(
                FILES,
                'conf',
                'cloud.providers.d',
                PROVIDER_NAME + '.conf'
            )
        )

        api_token = config[profile_str][DRIVER_NAME]['api_token']
        if api_token == '':
            self.skipTest(
                'api_token must be provided to '
                'run these tests. Check '
                'tests/integration/files/conf/cloud.providers.d/{0}.conf'
                .format(PROVIDER_NAME)
            )

    def test_list_images(self):
        '''
        Tests the return of running the --list-images command for 1and1
        '''
        image_list = self.run_cloud('--list-images {0}'.format(PROVIDER_NAME))
        self.assertIn(
            'coreOSimage',
            [i.strip() for i in image_list]
        )

    def test_instance(self):
        '''
        Test creating an instance on 1and1
        '''
        # check if instance with salt installed returned
        try:
            self.assertIn(
                INSTANCE_NAME,
                [i.strip() for i in self.run_cloud(
                    '-p oneandone-test {0}'.format(INSTANCE_NAME), timeout=500
                )]
            )
        except AssertionError:
            self.run_cloud('-d {0} --assume-yes'.format(INSTANCE_NAME), timeout=500)
            raise

        # delete the instance
        try:
            self.assertIn(
                INSTANCE_NAME + ':',
                [i.strip() for i in self.run_cloud(
                    '-d {0} --assume-yes'.format(INSTANCE_NAME), timeout=500
                )]
            )
        except AssertionError:
            raise

    def tearDown(self):
        '''
        Clean up after tests
        '''
        query = self.run_cloud('--query')
        ret = '        {0}:'.format(INSTANCE_NAME)

        # if test instance is still present, delete it
        if ret in query:
            self.run_cloud('-d {0} --assume-yes'.format(INSTANCE_NAME), timeout=500)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Mike Place <mp@saltstack.com>
'''

# Import Python libs
from __future__ import absolute_import, unicode_literals, print_function
from collections import namedtuple
import time

# Import Salt Testing libs
from tests.support.unit import TestCase, skipIf
from tests.support.mock import MagicMock, patch, call, Mock

# Import Salt libs
import salt.utils.data
import salt.modules.ps as ps

HAS_PSUTIL_VERSION = False

# Import 3rd-party libs
# pylint: disable=import-error,unused-import
from salt.ext.six.moves import range  # pylint: disable=redefined-builtin
import salt.utils.psutil_compat as psutil

PSUTIL2 = psutil.version_info >= (2, 0)

STUB_CPU_TIMES = namedtuple('cputimes', 'user nice system idle')(1, 2, 3, 4)
STUB_VIRT_MEM = namedtuple('vmem', 'total available percent used free')(1000, 500, 50, 500, 500)
STUB_SWAP_MEM = namedtuple('swap', 'total used free percent sin sout')(1000, 500, 500, 50, 0, 0)
STUB_PHY_MEM_USAGE = namedtuple('usage', 'total used free percent')(1000, 500, 500, 50)
STUB_DISK_PARTITION = namedtuple(
    'partition',
    'device mountpoint fstype, opts')(
        '/dev/disk0s2', '/', 'hfs', 'rw,local,rootfs,dovolfs,journaled,multilabel')
STUB_DISK_USAGE = namedtuple('usage', 'total used free percent')(1000, 500, 500, 50)
STUB_NETWORK_IO = namedtuple(
    'iostat',
    'bytes_sent, bytes_recv, packets_sent, packets_recv, errin errout dropin dropout')(
        1000, 2000, 500, 600, 1, 2, 3, 4)
STUB_DISK_IO = namedtuple(
    'iostat',
    'read_count, write_count, read_bytes, write_bytes, read_time, write_time')(
        1000, 2000, 500, 600, 2000, 3000)
STUB_USER = namedtuple('user', 'name, terminal, host, started')('bdobbs', 'ttys000', 'localhost', 0.0)
if psutil.version_info >= (0, 6, 0):
    HAS_PSUTIL_VERSION = True

STUB_PID_LIST = [0, 1, 2, 3]

try:
    import utmp  # pylint: disable=W0611

    HAS_UTMP = True
except ImportError:
    HAS_UTMP = False
# pylint: enable=import-error,unused-import


def _get_proc_name(proc):
    return proc.name() if PSUTIL2 else proc.name


def _get_proc_pid(proc):
    return proc.pid


class DummyProcess(object):
    '''
    Dummy class to emulate psutil.Process. This ensures that _any_ string
    values used for any of the options passed in are converted to str types on
    both Python 2 and Python 3.
    '''
    def __init__(self, cmdline=None, create_time=None, name=None, status=None,
                 username=None, pid=None):
        self._cmdline = salt.utils.data.decode(
            cmdline if cmdline is not None else [],
            to_str=True)
        self._create_time = salt.utils.data.decode(
            create_time if create_time is not None else time.time(),
            to_str=True)
        self._name = salt.utils.data.decode(
            name if name is not None else [],
            to_str=True)
        self._status = salt.utils.data.decode(status, to_str=True)
        self._username = salt.utils.data.decode(username, to_str=True)
        self._pid = salt.utils.data.decode(
            pid if pid is not None else 12345,
            to_str=True)

    def cmdline(self):
        return self._cmdline

    def create_time(self):
        return self._create_time

    def name(self):
        return self._name

    def status(self):
        return self._status

    def username(self):
        return self._username

    def pid(self):
        return self._pid


class PsTestCase(TestCase):
    def setUp(self):
        self.mocked_proc = mocked_proc = MagicMock('salt.utils.psutil_compat.Process')
        if PSUTIL2:
            self.mocked_proc.name = Mock(return_value="test_mock_proc")
            self.mocked_proc.pid = Mock(return_value=9999999999)
        else:
            self.mocked_proc.name = 'test_mock_proc'
            self.mocked_proc.pid = 9999999999

    @skipIf(not ps.PSUTIL2, 'Only run for psutil 2.x')
    def test__get_proc_cmdline(self):
        cmdline = ['echo', 'питон']
        ret = ps._get_proc_cmdline(DummyProcess(cmdline=cmdline))
        assert ret == cmdline, ret

    def test_get_pid_list(self):
        with patch('salt.utils.psutil_compat.pids',
                   MagicMock(return_value=STUB_PID_LIST)):
            self.assertListEqual(STUB_PID_LIST, ps.get_pid_list())

    def test_kill_pid(self):
        with patch('salt.utils.psutil_compat.Process') as send_signal_mock:
            ps.kill_pid(0, signal=999)
            self.assertEqual(send_signal_mock.call_args, call(0))

    def test_pkill(self):
        with patch('salt.utils.psutil_compat.Process.send_signal'), \
                patch('salt.utils.psutil_compat.process_iter',
                      MagicMock(return_value=[self.mocked_proc])):
            self.mocked_proc.send_signal = MagicMock()
            test_signal = 1234
            ps.pkill(_get_proc_name(self.mocked_proc), signal=test_signal)
            self.assertEqual(self.mocked_proc.send_signal.call_args, call(test_signal))

    def test_pgrep(self):
        with patch('salt.utils.psutil_compat.process_iter',
                   MagicMock(return_value=[self.mocked_proc])):
            self.assertIn(_get_proc_pid(self.mocked_proc), ps.pgrep(_get_proc_name(self.mocked_proc)))

    def test_cpu_percent(self):
        with patch('salt.utils.psutil_compat.cpu_percent',
                   MagicMock(return_value=1)):
            self.assertEqual(ps.cpu_percent(), 1)

    def test_cpu_times(self):
        with patch('salt.utils.psutil_compat.cpu_times',
                   MagicMock(return_value=STUB_CPU_TIMES)):
            self.assertDictEqual({'idle': 4, 'nice': 2, 'system': 3, 'user': 1}, ps.cpu_times())

    @skipIf(HAS_PSUTIL_VERSION is False, 'psutil 0.6.0 or greater is required for this test')
    def test_virtual_memory(self):
        with patch('salt.utils.psutil_compat.virtual_memory',
                   MagicMock(return_value=STUB_VIRT_MEM)):
            self.assertDictEqual({'used': 500, 'total': 1000, 'available': 500, 'percent': 50, 'free': 500},
                                 ps.virtual_memory())

    @skipIf(HAS_PSUTIL_VERSION is False, 'psutil 0.6.0 or greater is required for this test')
    def test_swap_memory(self):
        with patch('salt.utils.psutil_compat.swap_memory',
                   MagicMock(return_value=STUB_SWAP_MEM)):
            self.assertDictEqual({'used': 500, 'total': 1000, 'percent': 50, 'free': 500, 'sin': 0, 'sout': 0},
                                 ps.swap_memory())

    def test_disk_partitions(self):
        with patch('salt.utils.psutil_compat.disk_partitions',
                   MagicMock(return_value=[STUB_DISK_PARTITION])):
            self.assertDictEqual(
                {'device': '/dev/disk0s2', 'mountpoint': '/', 'opts': 'rw,local,rootfs,dovolfs,journaled,multilabel',
                 'fstype': 'hfs'},
                ps.disk_partitions()[0])

    def test_disk_usage(self):
        with patch('salt.utils.psutil_compat.disk_usage',
                   MagicMock(return_value=STUB_DISK_USAGE)):
            self.assertDictEqual({'used': 500, 'total': 1000, 'percent': 50, 'free': 500}, ps.disk_usage('DUMMY_PATH'))

    def test_disk_partition_usage(self):
        with patch('salt.utils.psutil_compat.disk_partitions',
                   MagicMock(return_value=[STUB_DISK_PARTITION])):
            self.assertDictEqual(
                {'device': '/dev/disk0s2', 'mountpoint': '/', 'opts': 'rw,local,rootfs,dovolfs,journaled,multilabel',
                 'fstype': 'hfs'},
                ps.disk_partitions()[0])

    def test_network_io_counters(self):
        with patch('salt.utils.psutil_compat.net_io_counters',
                   MagicMock(return_value=STUB_NETWORK_IO)):
            self.assertDictEqual(
                {'packets_sent': 500, 'packets_recv': 600, 'bytes_recv': 2000, 'dropout': 4, 'bytes_sent': 1000,
                 'errout': 2, 'errin': 1, 'dropin': 3}, ps.network_io_counters())

    def test_disk_io_counters(self):
        with patch('salt.utils.psutil_compat.disk_io_counters',
                   MagicMock(return_value=STUB_DISK_IO)):
            self.assertDictEqual(
                {'read_time': 2000, 'write_bytes': 600, 'read_bytes': 500, 'write_time': 3000, 'read_count': 1000,
                 'write_count': 2000}, ps.disk_io_counters())

    def test_get_users(self):
        with patch('salt.utils.psutil_compat.users',
                   MagicMock(return_value=[STUB_USER])):
            self.assertDictEqual({'terminal': 'ttys000', 'started': 0.0, 'host': 'localhost', 'name': 'bdobbs'},
                                 ps.get_users()[0])

        ## This is commented out pending discussion on https://github.com/saltstack/salt/commit/2e5c3162ef87cca8a2c7b12ade7c7e1b32028f0a
        # @skipIf(not HAS_UTMP, "The utmp module must be installed to run test_get_users_utmp()")
        # @patch('salt.utils.psutil_compat.get_users', new=MagicMock(return_value=None))  # This will force the function to use utmp
        # def test_get_users_utmp(self):
        #     pass



# -*- coding: utf-8 -*-

# Import Python Libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.utils.platform
import salt.utils.win_functions as win_functions


@skipIf(NO_MOCK, NO_MOCK_REASON)
class WinFunctionsTestCase(TestCase):
    '''
    Test cases for salt.utils.win_functions
    '''

    def test_escape_argument_simple(self):
        '''
        Test to make sure we encode simple arguments correctly
        '''
        encoded = win_functions.escape_argument('simple')

        self.assertEqual(encoded, 'simple')

    def test_escape_argument_with_space(self):
        '''
        Test to make sure we encode arguments containing spaces correctly
        '''
        encoded = win_functions.escape_argument('with space')

        self.assertEqual(encoded, '^"with space^"')

    def test_escape_argument_simple_path(self):
        '''
        Test to make sure we encode simple path arguments correctly
        '''
        encoded = win_functions.escape_argument('C:\\some\\path')

        self.assertEqual(encoded, 'C:\\some\\path')

    def test_escape_argument_path_with_space(self):
        '''
        Test to make sure we encode path arguments containing spaces correctly
        '''
        encoded = win_functions.escape_argument('C:\\Some Path\\With Spaces')

        self.assertEqual(encoded, '^"C:\\Some Path\\With Spaces^"')

    @skipIf(not salt.utils.platform.is_windows(), 'WinDLL only available on Windows')
    def test_broadcast_setting_change(self):
        '''
            Test to rehash the Environment variables
        '''
        self.assertTrue(win_functions.broadcast_setting_change())



# -*- coding: utf-8 -*-
'''
    :codeauthor: Nicole Thomas <nicole@saltstack.com>
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import yaml

# Import Salt Libs
from salt.config import cloud_providers_config
import salt.utils.cloud
import salt.utils.files
import salt.utils.yaml

# Import Salt Testing Libs
from tests.support.case import ShellCase
from tests.support.paths import FILES
from tests.support.helpers import expensiveTest, generate_random_name
from tests.support.unit import expectedFailure, skipIf
from tests.support import win_installer


# Create the cloud instance name to be used throughout the tests
INSTANCE_NAME = generate_random_name('CLOUD-TEST-')
PROVIDER_NAME = 'ec2'
HAS_WINRM = salt.utils.cloud.HAS_WINRM and salt.utils.cloud.HAS_SMB
TIMEOUT = 1200


class EC2Test(ShellCase):
    '''
    Integration tests for the EC2 cloud provider in Salt-Cloud
    '''
    def _installer_name(self):
        '''
        Determine the downloaded installer name by searching the files
        directory for the firt file that loosk like an installer.
        '''
        for path, dirs, files in os.walk(FILES):
            for file in files:
                if file.startswith(win_installer.PREFIX):
                    return file
            break
        return

    def _fetch_latest_installer(self):
        '''
        Download the latest Windows installer executable
        '''
        name = win_installer.latest_installer_name()
        path = os.path.join(FILES, name)
        with salt.utils.files.fopen(path, 'wb') as fp:
            win_installer.download_and_verify(fp, name)
        return name

    def _ensure_installer(self):
        '''
        Make sure the testing environment has a Windows installer executbale.
        '''
        name = self._installer_name()
        if name:
            return name
        return self._fetch_latest_installer()

    @expensiveTest
    def setUp(self):
        '''
        Sets up the test requirements
        '''
        super(EC2Test, self).setUp()

        # check if appropriate cloud provider and profile files are present
        profile_str = 'ec2-config'
        providers = self.run_cloud('--list-providers')

        if profile_str + ':' not in providers:
            self.skipTest(
                'Configuration file for {0} was not found. Check {0}.conf files '
                'in tests/integration/files/conf/cloud.*.d/ to run these tests.'
                .format(PROVIDER_NAME)
            )

        # check if id, key, keyname, securitygroup, private_key, location,
        # and provider are present
        config = cloud_providers_config(
            os.path.join(
                FILES,
                'conf',
                'cloud.providers.d',
                PROVIDER_NAME + '.conf'
            )
        )

        id_ = config[profile_str][PROVIDER_NAME]['id']
        key = config[profile_str][PROVIDER_NAME]['key']
        key_name = config[profile_str][PROVIDER_NAME]['keyname']
        sec_group = config[profile_str][PROVIDER_NAME]['securitygroupname'][0]
        private_key = config[profile_str][PROVIDER_NAME]['private_key']
        location = config[profile_str][PROVIDER_NAME]['location']

        conf_items = [id_, key, key_name, sec_group, private_key, location]
        missing_conf_item = []

        for item in conf_items:
            if item == '':
                missing_conf_item.append(item)

        if missing_conf_item:
            self.skipTest(
                'An id, key, keyname, security group, private key, and location must '
                'be provided to run these tests. One or more of these elements is '
                'missing. Check tests/integration/files/conf/cloud.providers.d/{0}.conf'
                .format(PROVIDER_NAME)
            )
        self.INSTALLER = self._ensure_installer()

    def override_profile_config(self, name, data):
        conf_path = os.path.join(self.get_config_dir(), 'cloud.profiles.d', 'ec2.conf')
        with salt.utils.files.fopen(conf_path, 'r') as fp:
            conf = yaml.safe_load(fp)
        conf[name].update(data)
        with salt.utils.files.fopen(conf_path, 'w') as fp:
            salt.utils.yaml.safe_dump(conf, fp)

    def copy_file(self, name):
        '''
        Copy a file from tests/integration/files to a test's temporary
        configuration directory. The path to the file which is created will be
        returned.
        '''
        src = os.path.join(FILES, name)
        dst = os.path.join(self.get_config_dir(), name)
        with salt.utils.files.fopen(src, 'rb') as sfp:
            with salt.utils.files.fopen(dst, 'wb') as dfp:
                dfp.write(sfp.read())
        return dst

    def _test_instance(self, profile='ec2-test', debug=False, timeout=TIMEOUT):
        '''
        Tests creating and deleting an instance on EC2 (classic)
        '''

        # create the instance
        cmd = '-p {0}'.format(profile)
        if debug:
            cmd += ' -l debug'
        cmd += ' {0}'.format(INSTANCE_NAME)
        instance = self.run_cloud(cmd, timeout=timeout)
        ret_str = '{0}:'.format(INSTANCE_NAME)

        # check if instance returned with salt installed
        try:
            self.assertIn(ret_str, instance)
        except AssertionError:
            self.run_cloud('-d {0} --assume-yes'.format(INSTANCE_NAME), timeout=timeout)
            raise

        # delete the instance
        delete = self.run_cloud('-d {0} --assume-yes'.format(INSTANCE_NAME), timeout=timeout)
        ret_str = '                    shutting-down'

        # check if deletion was performed appropriately
        try:
            self.assertIn(ret_str, delete)
        except AssertionError:
            raise

    def test_instance_rename(self):
        '''
        Tests creating and renaming an instance on EC2 (classic)
        '''
        # create the instance
        rename = INSTANCE_NAME + '-rename'
        instance = self.run_cloud('-p ec2-test {0} --no-deploy'.format(INSTANCE_NAME), timeout=TIMEOUT)
        ret_str = '{0}:'.format(INSTANCE_NAME)

        # check if instance returned
        try:
            self.assertIn(ret_str, instance)
        except AssertionError:
            self.run_cloud('-d {0} --assume-yes'.format(INSTANCE_NAME), timeout=TIMEOUT)
            raise

        change_name = self.run_cloud('-a rename {0} newname={1} --assume-yes'.format(INSTANCE_NAME, rename), timeout=TIMEOUT)

        check_rename = self.run_cloud('-a show_instance {0} --assume-yes'.format(rename), [rename])
        exp_results = ['        {0}:'.format(rename), '            size:',
                       '            architecture:']
        try:
            for result in exp_results:
                self.assertIn(result, check_rename[0])
        except AssertionError:
            self.run_cloud('-d {0} --assume-yes'.format(INSTANCE_NAME), timeout=TIMEOUT)
            raise

        # delete the instance
        delete = self.run_cloud('-d {0} --assume-yes'.format(rename), timeout=TIMEOUT)
        ret_str = '                    shutting-down'

        # check if deletion was performed appropriately
        self.assertIn(ret_str, delete)

    def test_instance(self):
        '''
        Tests creating and deleting an instance on EC2 (classic)
        '''
        self._test_instance('ec2-test')

    @expectedFailure
    def test_win2012r2_winexe(self):
        '''
        Tests creating and deleting a Windows 2012r2instance on EC2 using
        winexe (classic)
        '''
        # TODO: winexe calls hang and the test fails by timing out. The same
        # same calls succeed when run outside of the test environment.
        self.override_profile_config(
            'ec2-win2012r2-test',
            {
                'use_winrm': False,
                'userdata_file': self.copy_file('windows-firewall-winexe.ps1'),
                'win_installer': self.copy_file(self.INSTALLER),
            },
        )
        self._test_instance('ec2-win2012r2-test', debug=True, timeout=TIMEOUT)

    @skipIf(not HAS_WINRM, 'Skip when winrm dependencies are missing')
    def test_win2012r2_winrm(self):
        '''
        Tests creating and deleting a Windows 2012r2 instance on EC2 using
        winrm (classic)
        '''
        self.override_profile_config(
            'ec2-win2012r2-test',
            {
                'userdata_file': self.copy_file('windows-firewall.ps1'),
                'win_installer': self.copy_file(self.INSTALLER),
                'winrm_ssl_verify': False,
                'use_winrm': True,
            }

        )
        self._test_instance('ec2-win2012r2-test', debug=True, timeout=TIMEOUT)

    @expectedFailure
    def test_win2016_winexe(self):
        '''
        Tests creating and deleting a Windows 2016 instance on EC2 using winrm
        (classic)
        '''
        # TODO: winexe calls hang and the test fails by timing out. The same
        # same calls succeed when run outside of the test environment.
        self.override_profile_config(
            'ec2-win2016-test',
            {
                'use_winrm': False,
                'userdata_file': self.copy_file('windows-firewall-winexe.ps1'),
                'win_installer': self.copy_file(self.INSTALLER),
            },
        )
        self._test_instance('ec2-win2016-test', debug=True, timeout=TIMEOUT)

    @skipIf(not HAS_WINRM, 'Skip when winrm dependencies are missing')
    def test_win2016_winrm(self):
        '''
        Tests creating and deleting a Windows 2016 instance on EC2 using winrm
        (classic)
        '''
        self.override_profile_config(
            'ec2-win2016-test',
            {
                'userdata_file': self.copy_file('windows-firewall.ps1'),
                'win_installer': self.copy_file(self.INSTALLER),
                'winrm_ssl_verify': False,
                'use_winrm': True,
            }

        )
        self._test_instance('ec2-win2016-test', debug=True, timeout=TIMEOUT)

    def tearDown(self):
        '''
        Clean up after tests
        '''
        query = self.run_cloud('--query')
        ret_str = '        {0}:'.format(INSTANCE_NAME)

        # if test instance is still present, delete it
        if ret_str in query:
            self.run_cloud('-d {0} --assume-yes'.format(INSTANCE_NAME), timeout=TIMEOUT)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''
# Import Python libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch)

# Import Salt Libs
import salt.modules.system as system


@skipIf(NO_MOCK, NO_MOCK_REASON)
class SystemTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.system
    '''
    def setup_loader_modules(self):
        return {system: {}}

    def test_halt(self):
        '''
        Test to halt a running system
        '''
        with patch.dict(system.__salt__,
                        {'cmd.run': MagicMock(return_value='A')}):
            self.assertEqual(system.halt(), 'A')

    def test_init(self):
        '''
        Test to change the system runlevel on sysV compatible systems
        '''
        with patch.dict(system.__salt__,
                        {'cmd.run': MagicMock(return_value='A')}):
            self.assertEqual(system.init('r'), 'A')

    def test_poweroff(self):
        '''
        Test to poweroff a running system
        '''
        with patch.dict(system.__salt__,
                        {'cmd.run': MagicMock(return_value='A')}):
            self.assertEqual(system.poweroff(), 'A')

    def test_reboot(self):
        '''
        Test to reboot the system with shutdown -r
        '''
        cmd_mock = MagicMock(return_value='A')
        with patch.dict(system.__salt__, {'cmd.run': cmd_mock}):
            self.assertEqual(system.reboot(), 'A')
        cmd_mock.assert_called_with(['shutdown', '-r', 'now'], python_shell=False)

    def test_reboot_with_delay(self):
        '''
        Test to reboot the system using shutdown -r with a delay
        '''
        cmd_mock = MagicMock(return_value='A')
        with patch.dict(system.__salt__, {'cmd.run': cmd_mock}):
            self.assertEqual(system.reboot(at_time=5), 'A')
        cmd_mock.assert_called_with(['shutdown', '-r', '5'], python_shell=False)

    def test_shutdown(self):
        '''
        Test to shutdown a running system
        '''
        with patch.dict(system.__salt__,
                        {'cmd.run': MagicMock(return_value='A')}):
            self.assertEqual(system.shutdown(), 'A')



# -*- coding: utf-8 -*-
'''
unit tests for the cache runner
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.paths import TMP
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    patch
)

# Import Salt Libs
import salt.runners.cache as cache
import salt.utils.master


@skipIf(NO_MOCK, NO_MOCK_REASON)
class CacheTest(TestCase, LoaderModuleMockMixin):
    '''
    Validate the cache runner
    '''
    def setup_loader_modules(self):
        return {cache: {'__opts__': {'cache': 'localfs', 'pki_dir': TMP, 'key_cache': True}}}

    def test_grains(self):
        '''
        test cache.grains runner
        '''
        mock_minion = ['Larry']
        mock_ret = {}
        self.assertEqual(cache.grains(minion=mock_minion), mock_ret)

        mock_data = 'grain stuff'

        class MockMaster(object):
            def __init__(self, *args, **kwargs):
                pass

            def get_minion_grains(self):
                return mock_data

        with patch.object(salt.utils.master, 'MasterPillarUtil', MockMaster):
            self.assertEqual(cache.grains(), mock_data)
